/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file rhbdmc_compile.cpp
 * Program which aims to compile a given CNF into a RH-BDMC (Renamable Horn
 * BDMC). The idea is based on construction of a RH backdoor tree.
 */

#include <csignal>
#include <iostream>

#include "cpuutil.h"
#include "dimacs.h"
#include "logstream.h"
#include "prgutil.h"
#include "rh_bdmc.h"

const std::string SUBool::kPrgName = "rhbdmc_compile";
const std::string SUBool::kPrgDesc =
    "Program which aims to compile a given CNF into a RH-BDMC (Renamable Horn "
    "BDMC). The idea is based on construction of a RH backdoor tree. The "
    "program is *EXPERIMENTAL* and most probably does not work correctly.";

struct config_t : public SUBool::PrgConfigBase
{
   std::string input_file{};
   std::string output_file{};

   virtual void InnerDump(unsigned level) const override;
};

void
config_t::InnerDump(unsigned level) const
{
   SUBool::logs.DumpValue("\t", level, "input_file", input_file);
   SUBool::logs.DumpValue("\t", level, "output_file", output_file);
}

std::optional<config_t>
parse_arguments(int argc, char *argv[])
{
   config_t conf;
   SUBool::PrgOptions opts;
   opts.InputOutput(&conf.input_file, &conf.output_file);
   if (!opts.Parse(argc, argv, conf))
   {
      return {};
   }
   return conf;
}

void
term_func(int signal)
{
   SUBool::logs.TLog(0) << "Terminated by signal " << signal << std::endl;
   _Exit(signal);
}

int
main(int argc, char *argv[])
{
   auto conf = parse_arguments(argc, argv);
   if (!conf)
   {
      return 1;
   }
   std::signal(SIGTERM, term_func);
   std::signal(SIGQUIT, term_func);
   std::signal(SIGINT, term_func);
   conf->Dump();
   SUBool::RHBackdoorTree bt(
       SUBool::input_or_die<SUBool::CNF>(conf->input_file));
   SUBool::logs.TLog(1) << "Start compilation" << std::endl;
   SUBool::CPUTime cpu_time;
   bt.Compute();
   SUBool::logs.TLog(1) << "Compilation finished" << std::endl;
   SUBool::logs.TLog(1) << "Total processor time: " << cpu_time << std::endl;
   SUBool::logs.TLog(0) << "Number of leaves: " << bt.LeafSize() << std::endl;
   SUBool::output(conf->output_file, std::move(bt).CurrentNnf());
   SUBool::logs.Log(1) << std::endl;
   SUBool::logs.TLog(1) << "Output written" << std::endl;
   return 0;
}
