/*
 * Project SUBool
 * Petr Kucera, 2016
 */
/**@file rhornsplit.cpp
 * Program for splitting a CNF into renamable horn parts.
 */

#include <fstream>
#include <iostream>

#include "dimacs.h"
#include "horn.h"
#include "mrh.h"
#include "rhorn.h"

void
ProcessStream(std::istream &in, const std::string &name)
{
   auto cnf = SUBool::Dimacs::parse_cnf(in);
   std::vector<std::pair<SUBool::CNF, SUBool::Assignment>> part_list;
   SUBool::SplitToRenamableHornParts(cnf, &part_list);
   std::cout << "\"" << name << "\";";
   std::cout << cnf.MaxVariable() << ';' << cnf.Size() << ';'
             << part_list.size();
   for (unsigned i = 0; i < part_list.size() && i < 5; ++i)
   {
      std::cout << ';' << part_list[i].first.Size();
   }
   for (unsigned i = part_list.size(); i < 5; ++i)
   {
      std::cout << ";0";
   }
   std::vector<unsigned> var_count(cnf.MaxVariable(), 0);
   for (const auto &part : part_list)
   {
      std::vector<bool> var_in_part(cnf.MaxVariable(), false);
      for (const auto &clause : part.first)
      {
         for (const auto &lit : clause)
         {
            if (!var_in_part[lit.mVariable - 1])
            {
               var_in_part[lit.mVariable - 1] = true;
               ++var_count[lit.mVariable - 1];
            }
         }
      }
   }
   unsigned count = 0;
   for (const auto &c : var_count)
   {
      if (c > 1)
      {
         ++count;
      }
   }
   std::cout << ";" << count;
   cnf.ApplyRenaming(part_list.front().second);
   unsigned max_positive_size = 0;
   std::vector<bool> positive_vars(cnf.MaxVariable(), false);
   for (const auto &clause : cnf)
   {
      if (SUBool::IsHornClause(clause))
      {
         continue;
      }
      unsigned cnt = 0;
      for (const auto &lit : clause)
      {
         if (lit.Positive())
         {
            positive_vars[lit.Variable() - 1] = true;
            ++cnt;
         }
      }
      if (cnt > max_positive_size)
      {
         max_positive_size = cnt;
      }
   }
   std::cout << ";" << max_positive_size;
   unsigned positive_var_count = 0;
   for (unsigned i = 0; i < positive_vars.size(); ++i)
   {
      if (positive_vars[i])
      {
         ++positive_var_count;
      }
   }
   std::cout << ";" << positive_var_count;

   std::cout << std::endl;
}

int
main(int argc, char *argv[])
{
   std::cout
       << "name;max_var;cnf_size;n_parts;p0_size;p1_size;p2_size;p3_size;"
          "p4_size;common_vars;p0_max_positive_length;p0_positive_var_count"
       << std::endl;
   if (argc == 1)
   {
      ProcessStream(std::cin, "-");
   }
   else
   {
      for (int i = 1; i < argc; ++i)
      {
         std::ifstream in(argv[i]);
         ProcessStream(in, argv[i]);
      }
   }
   return 0;
}
