/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file absinccomp.h
 * Abstract iterative compiler. Common properties of an iterative compiler into
 * PC or UC formula.
 */

#ifndef __ABSINCCOMP_H
#define __ABSINCCOMP_H

#include <atomic>
#include <list>
#include <memory>
#include <vector>

#include "abscomp.h"
#include "comp_hypothesis.h"
#include "cpuutil.h"
#include "pcchecker.h"

namespace SUBool
{
   class AbsIncrementalComp : public AbsComp
   {
    public:
      static const StateValue FIND_EMPOWERING;
      static const StateValue INPROCESSING;
      static const StateValue PROCESS_LEARNED;

    protected:
      std::atomic<double> mTimeTillTheLastSATEqCheck{0.0};
      std::atomic<double> mTimeOfLastSATEqCheck{0.0};
      std::atomic<unsigned> mCntEmpowering;
      std::atomic<unsigned> mCntEmpoweringSAT;
      std::atomic<unsigned> mCntLearned;
      std::atomic<unsigned> mCntLearnedEmpowering;
      std::atomic<unsigned> mCntAdded; // excluding preprocessing
      unsigned mLastMinimizationCnt;   // Value of mCntAdded at the time of the
                                       // last minimization.
      CPUTime mCPUTime;
      RealTime mRTime;

      virtual CNF Inprocess(const Clause &emp, unsigned emp_var);
      virtual void AddEmpowering(const Clause &emp, unsigned emp_var);
      virtual const std::string &EmpowerMsg() = 0;

      virtual std::string PreambleGoal() const = 0;
      virtual std::string PreambleResultFormulaKind() const = 0;

      virtual std::string ResultDesc() const override;
      virtual std::string Annotation() const override;
      virtual void MinimizeHypothesis();
      virtual bool ShouldMinimize();

      virtual void CompilePreprocessed() override;
      virtual std::optional<Clause> FindEmpowering();

      virtual void ProcessLearned();

    public:
      AbsIncrementalComp(CNF cnf, MINIMIZE_METHOD minimize_method,
          IMPL_SYSTEM_REBUILD is_rebuild);
      virtual ~AbsIncrementalComp() = default;

      virtual void PrintStatistics(unsigned level) const override;
   };
}; // namespace SUBool

#endif
