/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file canon.h
 * Constructing canonical CNF.
 */

#include <list>
#include <queue>

#include "cpuutil.h"
#include "glucose_bind.h"
#include "normalform.h"

namespace SUBool
{

   class AbsCanon
   {
    public:
      static const size_t kClosedSizeLogStep = 1000;
      static const size_t kResolventCountLogStep = 10000;

    protected:
      size_t mResolventCount{0};
      CPUTime mCPUTime;

    public:
      AbsCanon() : mCPUTime() {}
      virtual ~AbsCanon() = default;
      virtual void Compute() = 0;
      virtual CNF Cnf() const = 0;
      virtual void PrintStatistics(unsigned level) const;

      size_t
      ResolventCount() const
      {
         return mResolventCount;
      }
   };

   class CanonBase : public AbsCanon
   {
      std::queue<Clause> mOpen;
      std::list<Clause> mClosed;
      CNF mInput;

      virtual bool SubsumedByClosed(const Clause &clause);

    public:
      CanonBase(CNF cnf);
      virtual ~CanonBase() = default;

      virtual void Compute() override;
      virtual CNF Cnf() const override;
   };

   class CanonSAT : public AbsCanon
   {
      std::queue<Clause> mOpen;
      std::vector<Clause> mClosed;
      CNF mInput;
      double mInprocTime{0.0};
      double mPreprocTime{0.0};
      IncSolverWrapper mSolver;

    public:
      CanonSAT(CNF cnf);

      virtual void Compute() override;
      virtual CNF Cnf() const override;
      virtual void PrintStatistics(unsigned level) const override;
   };

} // namespace SUBool
