/*
 * Project SUBool
 * Petr Kucera, 2021
 */
/**@file cardinal_opt.h
 * Program options related to cardinality constraints.
 */

#ifndef __CARDINAL_OPT_H
#define __CARDINAL_OPT_H

#include <boost/program_options.hpp>

#include "cardinal.h"
#include "enum_opt.h"
#include "prg_opt_group.h"

namespace po = boost::program_options;

namespace SUBool
{
   class CardinalOptionsGroup : public OptionsGroup
   {
    public:
      static constexpr AmoEncoder::KIND kDefaultAmoEncoding =
          SUBool::AmoEncoder::kDefaultGlobalKind;
      static constexpr ExOneEncoder::KIND kDefaultExOneEncoding =
          SUBool::ExOneEncoder::kDefaultGlobalKind;

    private:
      EnumOption<AmoEncoder::KIND> mAmoEncodingOption;
      EnumOption<ExOneEncoder::KIND> mExOneEncodingOption;

      AmoEncoder::KIND mAmoEncoding{kDefaultAmoEncoding};
      ExOneEncoder::KIND mExOneEncoding{kDefaultExOneEncoding};

    public:
      CardinalOptionsGroup();

      virtual bool GetValues() override;

      AmoEncoder::KIND AmoEncoding() const;
      ExOneEncoder::KIND ExOneEncoding() const;
   };
} // namespace SUBool

inline SUBool::AmoEncoder::KIND
SUBool::CardinalOptionsGroup::AmoEncoding() const
{
   return mAmoEncoding;
}

inline SUBool::ExOneEncoder::KIND
SUBool::CardinalOptionsGroup::ExOneEncoding() const
{
   return mExOneEncoding;
}
#endif
