/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file cnfcompress.h
 * Compressing CNF. This means renaming variables so that each
 * variable appears in some clause.
 */

#ifndef __CNFCOMPRESS_H
#define __CNFCOMPRESS_H

#include <mutex>
#include <vector>

#include "normalform.h"

namespace SUBool
{
   class CNFCompressor
   {
      mutable std::mutex mMutex;
      std::vector<unsigned> mVarMap;
      std::vector<unsigned> mVarBackmap;
      void InitVarMap(const CNF &cnf, unsigned max_fixed_variable);

      unsigned InnerMaxVariable() const;
      unsigned InnerUncompressVar(unsigned var) const;
      Literal InnerCompressLiteral(const Literal &lit) const;
      Literal InnerUncompressLiteral(const Literal &lit) const;
      unsigned InnerCompressVar(unsigned var) const;
      Clause InnerUncompressClause(const Clause &clause) const;
      Clause InnerCompressClause(const Clause &clause) const;

      std::pair<std::vector<unsigned>, std::vector<unsigned>> Maps() const &;
      std::pair<std::vector<unsigned>, std::vector<unsigned>> Maps() &;

    public:
      CNFCompressor() : CNFCompressor(CNF(), 0) {}
      CNFCompressor(const CNF &cnf, unsigned max_fixed_variable = 0);

      CNFCompressor(const CNFCompressor &other);
      CNFCompressor(CNFCompressor &&other);
      ~CNFCompressor() = default;
      CNFCompressor &operator=(const CNFCompressor &other);
      CNFCompressor &operator=(CNFCompressor &&other);

      unsigned MaxVariable() const;
      // Variables outside of the original CNF are unchanged
      unsigned UncompressVar(unsigned var) const;
      Literal UncompressLiteral(const Literal &lit) const;
      Literal CompressLiteral(const Literal &lit) const;
      // Variables outside of the original CNF are unchanged
      unsigned CompressVar(unsigned var) const;
      Clause UncompressClause(const Clause &clause) const;
      Clause CompressClause(const Clause &clause) const;

      CNF CompressCNF(CNF cnf) const;
      CNF UncompressCNF(CNF cnf) const;
   };
} // namespace SUBool

#endif
