/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file cpuutil.cpp
 * CPU time related functions.
 */

#include "cpuutil.h"

double
SUBool::cpu_time_since(const clock_t &c_start)
{
   clock_t c_end = clock();
   return (static_cast<double>(c_end - c_start)) / CLOCKS_PER_SEC;
}

SUBool::CPUTime::CPUTime() : mStart(clock()) {}

void
SUBool::CPUTime::Reset()
{
   mStart.store(clock());
}

double
SUBool::CPUTime::TimeSpan() const
{
   return cpu_time_since(mStart);
}

SUBool::RealTime::RealTime() : mStart(std::chrono::high_resolution_clock::now())
{
}

void
SUBool::RealTime::Reset()
{
   mStart.store(std::chrono::high_resolution_clock::now());
}

double
SUBool::RealTime::Duration() const
{
   auto t_end = std::chrono::high_resolution_clock::now();
   auto t_span = std::chrono::duration_cast<std::chrono::duration<double>>(
       t_end - mStart.load());
   return t_span.count();
}

std::ostream &
SUBool::operator<<(std::ostream &out, const CPUTime &cpu_time)
{
   return out << cpu_time.TimeSpan() << "s";
}

std::ostream &
SUBool::operator<<(std::ostream &out, const RealTime &real_time)
{
   return out << real_time.Duration() << "s";
}

