/*
 * Project SUBool
 * Petr Kucera, 2017
 */
/**@file glucose_bind.cpp
 * Binding class to glucose solver library. */

#include "glucose_bind.h"
#include "util.h"

Glucose::Lit
SUBool::glucose_lit(const Literal &lit)
{
   return Glucose::mkLit(lit.Variable() - 1, !lit.Positive());
}

SUBool::Literal
SUBool::of_glucose_lit(const Glucose::Lit &glit)
{
   return Literal(Glucose::var(glit) + 1, !Glucose::sign(glit));
}

SUBool::Clause
SUBool::of_glucose_clause(const Glucose::Clause &gclause)
{
   std::vector<Literal> literals;
   literals.reserve(gclause.size());
   for (int i = 0; i < gclause.size(); ++i)
   {
      literals.push_back(of_glucose_lit(gclause[i]));
   }
   return Clause(literals);
}

SUBool::PartialValue
SUBool::of_glucose_value(Glucose::lbool val)
{
   if (val == l_True)
   {
      return PartialValue::TRUE;
   }
   else if (val == l_False)
   {
      return PartialValue::FALSE;
   }
   return PartialValue::UNDEF;
}
