/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file implic_learn.cpp
 * Checking for implicates, prime implicates finding all with collecting learned
 * clauses.
 */

#include "implic_learn.h"

void
SUBool::ImplicateChecker::AddToLearnts(std::vector<Clause> clauses)
{
   for (auto &c : clauses)
   {
      mLearned.push(std::move(c));
   }
}

bool
SUBool::ImplicateChecker::IsImplicate(const CNF &cnf, const Clause &clause)
{
   SimpSolverWrapper wrap(cnf);
   auto &g = wrap.Solver();
   auto r = is_implicate(g, clause);
   AddToLearnts(g.Learnts());
   return r;
}

std::optional<SUBool::Clause>
SUBool::ImplicateChecker::PrimeSubimplicate(const CNF &cnf,
                                            const Clause &clause,
                                            unsigned ignore_var,
                                            bool promised_sat)
{
   IncSolverWrapper wrap(cnf);
   auto &g = wrap.Solver();
   auto prime = prime_subimplicate(cnf, clause, ignore_var, promised_sat);
   AddToLearnts(g.Learnts());
   return prime;
}

std::optional<SUBool::Clause>
SUBool::ImplicateChecker::NextLearnedClause()
{
   if (mLearned.empty())
   {
      return {};
   }
   auto r = std::make_optional(mLearned.top());
   mLearned.pop();
   return r;
}

bool
SUBool::ImplicateChecker::HasLearnedClause()
{
   return !mLearned.empty();
}

size_t
SUBool::ImplicateChecker::LearnedSize() const
{
   return mLearned.size();
}

bool
SUBool::ImplicateChecker::Solve(const CNF &cnf,
                                const std::vector<Literal> &assump)
{
   SimpSolverWrapper wrap(cnf);
   auto &g = wrap.Solver();
   auto result = g.Solve(assump);
   AddToLearnts(g.Learnts());
   return result;
}
