/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file implic_learn.h
 * Checking for implicates, prime implicates finding all with collecting learned
 * clauses.
 */

#ifndef __IMPLIC_LEARN_H
#define __IMPLIC_LEARN_H

#include "cnfutil.h"
#include "normalform.h"

namespace SUBool
{
   class ImplicateChecker
   {
      std::priority_queue<Clause, std::vector<Clause>, Clause::SizeCompare>
          mLearned;

      void AddToLearnts(std::vector<Clause> clauses);

    public:
      ImplicateChecker() : mLearned() {}

      bool IsImplicate(const CNF &cnf, const Clause &clause);
      std::optional<Clause> PrimeSubimplicate(const CNF &cnf,
                                              const Clause &clause,
                                              unsigned ignore_var = 0,
                                              bool promised_sat = false);
      bool Solve(const CNF &cnf, const std::vector<Literal> &assump);

      std::optional<Clause> NextLearnedClause();
      bool HasLearnedClause();
      size_t LearnedSize() const;
   };
} // namespace SUBool

#endif
