/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file logstream.h
 * A buffer for creating a stream which discards everything and stream
 * which.
 */

#ifndef __LOGSTREAM_H
#define __LOGSTREAM_H

#include <atomic>
#include <iomanip>
#include <iostream>

#include "nullstream.h"
#include "util.h"

namespace SUBool
{
   class LogStream
   {
      std::ostream *mOut;
      char mBuffer[64];

    public:
      enum class LOG_LEVEL
      {
         NONE,
         MINLINES,
         EMPOWER,
         ERROR = 1,
         ALL
      };
      static const char *kMaxLogLevels;
      static const unsigned kDefaultLogLevel = 3;
      static const size_t kMinLogCNFSize = 10000;
      static const size_t kLogStep = 1000;
      static const unsigned kDumpConfigLevel = 1;

      std::atomic<unsigned> mVerbLevel;

      LogStream(std::ostream &out, unsigned verbosity_level)
          : mOut(&out), mVerbLevel(verbosity_level)
      {
      }

      void Set(std::ostream &out, unsigned verbosity_level);

      std::ostream &Log(unsigned level);
      std::ostream &
      Log(LOG_LEVEL level)
      {
         return Log(static_cast<unsigned>(level));
      }

      // Starts a comment line
      std::ostream &CLog(unsigned level);
      std::ostream &
      CLog(LOG_LEVEL level)
      {
         return CLog(static_cast<unsigned>(level));
      }

      // Starts a comment line with a timestamp
      std::ostream &TLog(unsigned level);
      std::ostream &
      TLog(LOG_LEVEL level)
      {
         return TLog(static_cast<unsigned>(level));
      }

      bool
      CheckLevel(unsigned level) const
      {
         return level <= mVerbLevel;
      }

      template <class T>
      void
      DumpValue(const std::string &line_prefix, unsigned level,
          const std::string &name, const T &value)
      {
         if (CheckLevel(level))
         {
            TLog(level) << line_prefix << name << ": " << value << std::endl;
         }
      }

      void DumpVerbosity(const std::string &line_prefix, unsigned level);
   };

   extern LogStream logs;

}; // namespace SUBool

template <>
inline void
SUBool::LogStream::DumpValue<bool>(const std::string &line_prefix,
    unsigned level, const std::string &name, const bool &value)
{
   DumpValue<std::string>(line_prefix, level, name, bool_to_string(value));
}

inline void
SUBool::LogStream::DumpVerbosity(const std::string &line_prefix, unsigned level)
{
   DumpValue(line_prefix, level, "verbosity", mVerbLevel);
}

#endif
