/*
 * Project SUBool
 * Petr Kucera, 2016
 */
/**@file nfliteralset_impl.h
 * Contains implementation of function of template NFLiteralSet.
 */

#include <algorithm>
#include <sstream>

#include "sbexception.h"

template <bool DETERMINING_VALUE>
const SUBool::PartialValue
    SUBool::NFLiteralSet<DETERMINING_VALUE>::cDeterminingPartialValue;

template <bool DETERMINING_VALUE>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet() : mLiterals() {}

template <bool DETERMINING_VALUE>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet(Literal lit)
    : mLiterals() {
  mLiterals.push_back(std::move(lit));
  Normalize();
}

template <bool DETERMINING_VALUE>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet(Literal l1, Literal l2)
    : mLiterals{l1, l2} {
  Normalize();
}

template <bool DETERMINING_VALUE>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet(Literal l1, Literal l2,
                                                      Literal l3)
    : mLiterals{l1, l2, l3} {
  Normalize();
}

template <bool DETERMINING_VALUE>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet(
    std::initializer_list<Literal> il)
    : mLiterals(std::move(il)) {
  Normalize();
}

template <bool DETERMINING_VALUE>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet(
    std::vector<Literal> literals)
    : mLiterals(std::move(literals)) {
  Normalize();
}

template <bool DETERMINING_VALUE>
template <class InputIterator>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet(InputIterator begin,
                                                      InputIterator end)
    : mLiterals(begin, end) {
  Normalize();
}

template <bool DETERMINING_VALUE>
SUBool::NFLiteralSet<DETERMINING_VALUE>::NFLiteralSet(
    const PartialAssignment &assignment)
    : mLiterals() {
  for (unsigned i = 1; i <= assignment.Size(); ++i) {
    switch (assignment[i]) {
    case PartialValue::TRUE:
      mLiterals.emplace_back(i, !DETERMINING_VALUE);
      break;
    case PartialValue::FALSE:
      mLiterals.emplace_back(i, DETERMINING_VALUE);
      break;
    default:
      break;
    }
  }
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::DeterminingValue() {
  return DETERMINING_VALUE;
}

template <bool DETERMINING_VALUE>
void SUBool::NFLiteralSet<DETERMINING_VALUE>::Clear() {
  mLiterals.clear();
}

template <bool DETERMINING_VALUE>
size_t SUBool::NFLiteralSet<DETERMINING_VALUE>::Size() const {
  return mLiterals.size();
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::Empty() const {
  return mLiterals.empty();
}

template <bool DETERMINING_VALUE>
const SUBool::Literal &
SUBool::NFLiteralSet<DETERMINING_VALUE>::GetLiteral(size_t index) const {
  if (index >= mLiterals.size()) {
    throw OutOfBoundsException("index>=mLiterals.size()");
  }
  return mLiterals[index];
}

template <bool DETERMINING_VALUE>
const SUBool::Literal &
SUBool::NFLiteralSet<DETERMINING_VALUE>::operator[](size_t index) const {
  return mLiterals[index];
}

template <bool DETERMINING_VALUE>
const SUBool::Literal &SUBool::NFLiteralSet<DETERMINING_VALUE>::Front() const {
  return mLiterals.front();
}

template <bool DETERMINING_VALUE>
const SUBool::Literal &SUBool::NFLiteralSet<DETERMINING_VALUE>::Back() const {
  return mLiterals.back();
}

template <bool DETERMINING_VALUE>
const SUBool::Literal &
SUBool::NFLiteralSet<DETERMINING_VALUE>::LiteralOnVariable(unsigned var) const {
  const auto lb =
      std::lower_bound(mLiterals.begin(), mLiterals.end(), Literal(var, true));
  if (lb->Variable() != var) {
    throw NotFoundException("LiteralOnVariable",
                            "Variable not found in the element set.");
  }
  return *lb;
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::Value(
    const Assignment &assignment) const {
  for (const auto &lit : mLiterals) {
    // std::cout<<lit.mVariable<<" "<<lit.mPositive<<"
    // "<<assignment[lit.mVariable]<<"
    // "<<lit.Value(assignment[lit.mVariable])<<std::endl;
    if (lit.Value(assignment) == DETERMINING_VALUE) {
      return DETERMINING_VALUE;
    }
  }
  return !DETERMINING_VALUE;
}

template <bool DETERMINING_VALUE>
SUBool::PartialValue SUBool::NFLiteralSet<DETERMINING_VALUE>::Value(
    const PartialAssignment &assignment) const {
  bool has_undef = false;
  for (const auto &lit : mLiterals) {
    switch (lit.Value(assignment)) {
    case cDeterminingPartialValue:
      return cDeterminingPartialValue;
    case PartialValue::UNDEF:
      has_undef = true;
      break;
    default:
      break;
    }
  }
  return (has_undef ? PartialValue::UNDEF : !cDeterminingPartialValue);
}

template <bool DETERMINING_VALUE>
SUBool::PartialValue SUBool::NFLiteralSet<DETERMINING_VALUE>::Apply(
    const PartialAssignment &assignment) {
  NFLiteralSet<DETERMINING_VALUE> new_element;
  PartialValue val = Apply(assignment, &new_element);
  *this = new_element;
  return val;
}

template <bool DETERMINING_VALUE>
SUBool::PartialValue SUBool::NFLiteralSet<DETERMINING_VALUE>::Apply(
    const PartialAssignment &assignment,
    NFLiteralSet<DETERMINING_VALUE> *new_element) const {
  if (new_element == nullptr) {
    throw NullPointerException("NFLiteralSet::Apply", "new_element == nullptr");
  }
  std::vector<Literal> new_literals;
  PartialValue val = !cDeterminingPartialValue;
  for (const auto &lit : mLiterals) {
    switch (lit.Value(assignment)) {
    case cDeterminingPartialValue:
      new_literals.clear();
      val = cDeterminingPartialValue;
      break;
    case PartialValue::UNDEF:
      new_literals.push_back(lit);
      val = PartialValue::UNDEF;
      break;
    default:
      break;
    }
    if (val == cDeterminingPartialValue) {
      break;
    }
  }
  *new_element = NFLiteralSet<DETERMINING_VALUE>(new_literals);
  return val;
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::operator==(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  if (ls.mLiterals.size() != mLiterals.size()) {
    return false;
  }
  auto mismatch_pair =
      std::mismatch(mLiterals.begin(), mLiterals.end(), ls.mLiterals.begin());
  return (mismatch_pair.first == mLiterals.end());
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::operator!=(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  if (ls.mLiterals.size() != mLiterals.size()) {
    return true;
  }
  auto mismatch_pair =
      std::mismatch(mLiterals.begin(), mLiterals.end(), ls.mLiterals.begin());
  return (mismatch_pair.first != mLiterals.end());
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::operator<(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  if (mLiterals.size() > ls.mLiterals.size()) {
    return false;
  }
  return mLiterals.size() < ls.mLiterals.size() ||
         std::lexicographical_compare(mLiterals.begin(), mLiterals.end(),
                                      ls.mLiterals.begin(), ls.mLiterals.end());
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::operator>(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  if (ls.mLiterals.size() > mLiterals.size()) {
    return false;
  }
  return ls.mLiterals.size() < mLiterals.size() ||
         std::lexicographical_compare(ls.mLiterals.begin(), ls.mLiterals.end(),
                                      mLiterals.begin(), mLiterals.end());
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::operator<=(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  return !(*this > ls);
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::operator>=(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  return !(*this < ls);
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::Subsumes(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  return std::includes(ls.mLiterals.begin(), ls.mLiterals.end(),
                       mLiterals.begin(), mLiterals.end());
}

template <bool DETERMINING_VALUE>
std::ostream &SUBool::NFLiteralSet<DETERMINING_VALUE>::WriteToStream(
    std::ostream &output) const {
  std::copy(mLiterals.begin(), mLiterals.end(),
            std::ostream_iterator<Literal>(output, " "));
  output << "0";
  return output;
}

template <bool DETERMINING_VALUE>
SUBool::SetCompareResult SUBool::NFLiteralSet<DETERMINING_VALUE>::SubsetCompare(
    const NFLiteralSet<DETERMINING_VALUE> &ls) const {
  return compare_sets(mLiterals, ls.mLiterals);
}

template <bool DETERMINING_VALUE>
unsigned SUBool::NFLiteralSet<DETERMINING_VALUE>::MaxVariable() const {
  return (mLiterals.empty() ? 0 : mLiterals.back().mVariable);
}

template <bool DETERMINING_VALUE>
std::pair<unsigned, unsigned>
SUBool::NFLiteralSet<DETERMINING_VALUE>::CountVariablePolarities() const {
  unsigned positive = 0;
  unsigned negative = 0;
  for (const auto &lit : mLiterals) {
    if (lit.mPositive) {
      ++positive;
    } else {
      ++negative;
    }
  }
  return std::make_pair(positive, negative);
}

template <bool DETERMINING_VALUE>
void SUBool::NFLiteralSet<DETERMINING_VALUE>::Normalize() {
  if (!std::is_sorted(mLiterals.begin(), mLiterals.end())) {
    std::sort(mLiterals.begin(), mLiterals.end());
  }
  mLiterals.erase(std::unique(mLiterals.begin(), mLiterals.end()),
                  mLiterals.end());
}

template <bool DETERMINING_VALUE>
bool SUBool::NFLiteralSet<DETERMINING_VALUE>::Trivial() const {
  bool have_complement = false;
  for (unsigned i = 1; !have_complement && i < mLiterals.size(); ++i) {
    if (mLiterals[i].mVariable == mLiterals[i - 1].mVariable) {
      have_complement = true;
    }
  }
  return have_complement;
}

template <bool DETERMINING_VALUE>
void SUBool::NFLiteralSet<DETERMINING_VALUE>::NegateLiterals() {
  for (auto &lit : mLiterals) {
    lit.Negate();
  }
}

template <bool DETERMINING_VALUE>
void SUBool::NFLiteralSet<DETERMINING_VALUE>::PushBack(Literal lit) {
  unsigned i = mLiterals.size();
  while (i > 0 && mLiterals[i - 1] > lit) {
    --i;
  }
  if (i > 0 && mLiterals[i - 1] == lit) {
    return;
  }
  for (; i < mLiterals.size(); ++i) {
    std::swap(lit, mLiterals[i]);
  }
  mLiterals.push_back(lit);
}

template <bool DETERMINING_VALUE>
void SUBool::NFLiteralSet<DETERMINING_VALUE>::Append(
    const NFLiteralSet<DETERMINING_VALUE> &ls) {
  mLiterals.reserve(Size() + ls.Size());
  for (const auto &lit : ls) {
    mLiterals.push_back(lit);
  }
}

template <bool DETERMINING_VALUE>
void SUBool::NFLiteralSet<DETERMINING_VALUE>::ApplyRenaming(
    const Assignment &renaming) {
  for (auto &lit : mLiterals) {
    if (renaming[lit.mVariable]) {
      lit.Negate();
    }
  }
}
