/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file nnfnodelist.cpp
 * List of NNF nodes. The list is in topological order and contains leaves at
 * the beginning, gates follow. This is useful for some algorithm manipulating
 * the NNF.
 */

#include "nnfnodelist.h"
#include "nnfvisit.h"

namespace SUBool
{
   class NnfNodeListVisitor : public NnfVisitorMut
   {
      NnfNodeList mNodes{{}, {}};

    public:
      virtual void VisitLiteral(const LiteralPtr &node) override;
      virtual void VisitAnd(const GateAndPtr &node) override;
      virtual void VisitOr(const GateOrPtr &node) override;
      virtual void VisitDecision(const GateDecisionPtr &node) override;
      virtual void VisitTrue(const TruePtr &node) override;
      virtual void VisitFalse(const FalsePtr &node) override;

      NnfNodeList Nodes() const &;
      NnfNodeList Nodes() &&;
   };
} // namespace SUBool

void
SUBool::NnfNodeListVisitor::VisitLiteral(const LiteralPtr &node)
{
   mNodes.leaves.push_back(node);
}

void
SUBool::NnfNodeListVisitor::VisitAnd(const GateAndPtr &node)
{
   mNodes.gates.push_back(node);
}

void
SUBool::NnfNodeListVisitor::VisitOr(const GateOrPtr &node)
{
   mNodes.gates.push_back(node);
}

void
SUBool::NnfNodeListVisitor::VisitDecision(const GateDecisionPtr &node)
{
   mNodes.gates.push_back(node);
}

void
SUBool::NnfNodeListVisitor::VisitTrue(const TruePtr &node)
{
   mNodes.leaves.push_back(node);
}

void
SUBool::NnfNodeListVisitor::VisitFalse(const FalsePtr &node)
{
   mNodes.leaves.push_back(node);
}

SUBool::NnfNodeList
SUBool::NnfNodeListVisitor::Nodes() const &
{
   return mNodes;
}

SUBool::NnfNodeList
SUBool::NnfNodeListVisitor::Nodes() &&
{
   return std::move(mNodes);
}

SUBool::NnfNodeList
SUBool::nnf_node_list(Nnf nnf)
{
   NnfNodeListVisitor visitor;
   nnf.AcceptPostorderMut(visitor);
   return std::move(visitor).Nodes();
}
