/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file nnfnodelist.h
 * List of NNF nodes. The list is in topological order and contains leaves at
 * the beginning, gates follow. This is useful for some algorithm manipulating
 * the NNF.
 */

#ifndef __NNFNODELIST_H
#define __NNFNODELIST_H

#include "nnf.h"

namespace SUBool
{

   // A representation of a NNF as a list of nodes in a topological order.
   // The first leaf_size nodes are leaves, then gates follow in a topological
   // order, the last node is root.
   struct NnfNodeList
   {
      std::vector<std::shared_ptr<NnfLeaf>> leaves;
      std::vector<std::shared_ptr<NnfGate>> gates;
   };

   // Produces a node list for a given nnf.
   // Nnf is passed by value because it is unsafe to use it as a NNF when the
   // nodes can be modified and accessed from the list at the same time.
   NnfNodeList nnf_node_list(Nnf nnf);

} // namespace SUBool

#endif
