/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file pccomp.h
 * A compiler into a propagation complete formula (PC).
 */

#include "pccomp.h"
#include "cnfutil.h"
#include "logstream.h"

const std::string SUBool::PCComp::kEmpowerMsg =
    "Found empowering implicate with empowering variable";

SUBool::PCComp::PCComp(CNF cnf, Config conf)
    : AbsIncrementalComp(
        std::move(cnf), conf.minimize_method, conf.impl_system_rebuild),
      mCheckerSAT(), mEncConf(conf.enc_conf), mConf(std::move(conf))
{
   mCheckerSAT.Init(mEncConf, true, mConf.timeouts);
   mCheckerSAT.SetSolverType(mConf.solver_type);
}

const SUBool::AbsComp::BaseConfig &
SUBool::PCComp::GetBaseConfig() const
{
   return mConf;
}

const std::string &
SUBool::PCComp::EmpowerMsg()
{
   return kEmpowerMsg;
}

SUBool::AbsCheckerSAT &
SUBool::PCComp::CheckerSAT()
{
   return mCheckerSAT;
}

const SUBool::AbsCheckerSAT &
SUBool::PCComp::CheckerSAT() const
{
   return mCheckerSAT;
}

std::string
SUBool::PCComp::PreambleGoal() const
{
   return "PC";
}

std::string
SUBool::PCComp::PreambleResultFormulaKind() const
{
   return "propagation complete";
}
