/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file pcl_negative.cpp
 * Negative examples maintenance and storage.
 *
 * The compiler of a CNF into PC representation which is based on the
 * algorithm for learning of Horn formulas through equivalence and closure
 * queries (Arias et al. 2015).
 */

#include <algorithm>

#include "drenc.h"
#include "pcl_negative.h"
#include "sbexception.h"

std::ostream &
SUBool::operator<<(std::ostream &out, const SUBool::NEPrinter &printer)
{
   if (printer.print_as_clauses)
   {
      for (const auto &clause : dr_clauses_of_impl(printer.neg))
      {
         out << printer.compressor.UncompressClause(clause) << std::endl;
      }
      return out;
   }
   printer.neg.PrettyPrint(out, [&printer](const Literal &lit) {
      return printer.compressor.UncompressLiteral(lit);
   });
   return out;
   // return printer.neg.Print(out, printer.compressor);
}

std::optional<SUBool::LitImpl>
SUBool::ne_proper_intersection(const LitImpl &a, const LitImpl &b)
{
   std::vector<Literal> intersect_literals;
   std::set_intersection(a.Body().cbegin(), a.Body().cend(), b.Body().cbegin(),
                         b.Body().cend(),
                         std::back_inserter(intersect_literals));
   if (intersect_literals.empty() || intersect_literals.size() == b.BodySize())
   {
      return {};
   }
   return std::make_optional(LitImpl(intersect_literals, {}, true));
}

SUBool::LitImpl
SUBool::proto_negative_of_clause(const Clause &clause, const Literal &emp)
{
   std::vector<Literal> body;
   body.reserve(clause.Size() - 1);
   for (const auto &lit : clause)
   {
      if (lit != emp)
      {
         body.push_back(lit.Negation());
      }
   }
   return LitImpl(std::move(body), {emp}, true);
}

void
SUBool::add_proto_negatives_of_clause(std::vector<LitImpl> &negatives,
                                      const Clause &clause,
                                      const std::vector<Literal> &empowered)
{
   for (const auto &emp : empowered)
   {
      negatives.emplace_back(proto_negative_of_clause(clause, emp));
   }
}
