/*
 * Project SUBool
 * Petr Kucera, 2021
 */
/**@file prg_opt_group.h
 * Program options group.
 */

#ifndef __PRG_OPT_GROUP_H
#define __PRG_OPT_GROUP_H

#include <boost/program_options.hpp>
#include <boost/smart_ptr/make_shared.hpp>

namespace po = boost::program_options;

namespace SUBool
{

   class OptionsGroup
   {
      po::options_description mOptionsDesc;

    protected:
      po::options_description_easy_init AddOptions();
      void Add(boost::shared_ptr<po::option_description> opt);

    public:
      OptionsGroup(const std::string &name);
      virtual ~OptionsGroup() = default;

      virtual bool GetValues() = 0;
      const po::options_description &Desc() const;
   };

} // namespace SUBool

inline const po::options_description &
SUBool::OptionsGroup::Desc() const
{
   return mOptionsDesc;
}

inline po::options_description_easy_init
SUBool::OptionsGroup::AddOptions()
{
   return mOptionsDesc.add_options();
}

inline void
SUBool::OptionsGroup::Add(boost::shared_ptr<po::option_description> opt)
{
   mOptionsDesc.add(std::move(opt));
}

inline SUBool::OptionsGroup::OptionsGroup(const std::string &name)
    : mOptionsDesc(name)
{
}

#endif

