/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file random_utils.h
 * Contains common functions and structures used for generating random sequences
 * and objects.
 */

#ifndef __RANDOM_UTILS_H
#define __RANDOM_UTILS_H

#include <chrono>
#include <random>
#include <vector>

#include "literal.h"
#include "nfliteralset.h"
#include "normalform.h"

namespace SUBool
{
   // Initial seed at construction is the current time.
   extern std::mt19937_64 sub_rand;

   std::vector<Literal> random_literal_vector(const unsigned n_vars);
   std::vector<Literal> random_literal_vector_with_size(const unsigned size,
                                                        const unsigned n_vars);
   // random horn cnf and clauses are pure horn
   CNF random_horn_cnf(const unsigned size, const unsigned max_clause_size,
                       const unsigned n_vars, bool pure);
   Clause random_horn_clause(const unsigned size, const unsigned n_vars,
                             bool pure);
   CNF random_k_cnf(const unsigned size, const unsigned clause_size,
                    const unsigned n_vars);
   CNF random_cnf(const unsigned size, const unsigned n_vars);
   std::pair<CNF, Assignment> random_sat_cnf(const unsigned size,
                                             const unsigned n_vars);
} // namespace SUBool

#endif
