/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file resolve.h
 * Functions and structures for algorithms related to resolution.
 */

#include "resolve.h"

std::pair<SUBool::Clause, unsigned>
SUBool::resolve(const Clause &left, const Clause &right)
{
   unsigned res_var = 0;
   std::vector<Literal> res;
   auto l = left.cbegin();
   auto r = right.cbegin();
   while (l != left.cend() && r != right.cend())
   {
      if (l->Variable() < r->Variable())
      {
         res.push_back(*l);
         ++l;
         continue;
      }
      if (r->Variable() < l->Variable())
      {
         res.push_back(*r);
         ++r;
         continue;
      }
      if (*r == *l)
      {
         res.push_back(*r);
         ++r;
         ++l;
         continue;
      }
      if (res_var > 0)
      {
         return std::make_pair(Clause(), 0);
      }
      res_var = l->Variable();
      ++r;
      ++l;
   }
   while (l != left.cend())
   {
      res.push_back(*l);
      ++l;
   }
   while (r != right.cend())
   {
      res.push_back(*r);
      ++r;
   }
   return (res_var > 0 ? std::make_pair(std::move(res), res_var)
                       : std::make_pair(Clause(), 0));
}

bool
SUBool::resolvable(const Clause &left, const Clause &right)
{
   unsigned res_var = 0;
   auto l = left.cbegin();
   auto r = right.cbegin();
   while (l != left.cend() && r != right.cend())
   {
      if (l->Variable() < r->Variable())
      {
         ++l;
         continue;
      }
      if (r->Variable() < l->Variable())
      {
         ++r;
         continue;
      }
      if (*r == *l)
      {
         ++r;
         ++l;
         continue;
      }
      if (res_var > 0)
      {
         return false;
      }
      res_var = l->Variable();
      ++r;
      ++l;
   }
   return (res_var > 0);
}
