/*
 * Project SUBool
 * Petr Kucera, 2016
 */
/**@file sbexception.h
 * Exception classes.
 */

#ifndef __SBEXCEPTION_H
#define __SBEXCEPTION_H

#include <stdexcept>
#include <string>

namespace SUBool
{
   class SBException : public std::exception
   {
      std::string mWhatMsg;

    public:
      explicit SBException(const std::string &func,
                           const std::string &msg) throw()
          : mWhatMsg()
      {
         mWhatMsg += "[";
         mWhatMsg += func;
         mWhatMsg += "] ";
         mWhatMsg += msg;
      }
      explicit SBException(const std::string &what_arg) throw()
          : mWhatMsg(what_arg)
      {
      }
      virtual const char *
      what() const throw()
      {
         return mWhatMsg.c_str();
      }

      virtual ~SBException() throw() {}
   };

   class OutOfBoundsException : public SBException
   {
    public:
      explicit OutOfBoundsException(const std::string &func,
                                    const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit OutOfBoundsException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class IOException : public SBException
   {
    public:
      explicit IOException(const std::string &func, const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit IOException(const std::string &what_arg) : SBException(what_arg)
      {
      }
   };

   class BadParameterException : public SBException
   {
    public:
      explicit BadParameterException(const std::string &func,
                                     const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit BadParameterException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class BadInputException : public SBException
   {
    public:
      explicit BadInputException(const std::string &func,
                                 const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit BadInputException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class NullPointerException : public SBException
   {
    public:
      explicit NullPointerException(const std::string &func,
                                    const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit NullPointerException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class InvariantFailedException : public SBException
   {
    public:
      explicit InvariantFailedException(const std::string &func,
                                        const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit InvariantFailedException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class NoModelException : public SBException
   {
    public:
      explicit NoModelException(const std::string &func, const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit NoModelException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class NotFoundException : public SBException
   {
    public:
      explicit NotFoundException(const std::string &func,
                                 const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit NotFoundException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class ParseException : public SBException
   {
    public:
      explicit ParseException(const std::string &func, const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit ParseException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class NotImplementedException : public SBException
   {
    public:
      explicit NotImplementedException(const std::string &func,
                                       const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit NotImplementedException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class OverflowException : public SBException
   {
    public:
      explicit OverflowException(const std::string &func,
                                 const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit OverflowException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class FailureException : public SBException
   {
    public:
      explicit FailureException(const std::string &func, const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit FailureException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class SolverTerminatedException : public SBException
   {
    public:
      explicit SolverTerminatedException(const std::string &func,
                                         const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit SolverTerminatedException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

   class ConflictDetectedException : public SBException
   {
    public:
      explicit ConflictDetectedException(const std::string &func,
                                         const std::string &msg)
          : SBException(func, msg)
      {
      }
      explicit ConflictDetectedException(const std::string &what_arg)
          : SBException(what_arg)
      {
      }
   };

} // namespace SUBool

#endif
