/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file simple_constraints.h
 * Encodings of various constraints.
 */

#ifndef __SIMPLE_CONSTRAINTS_H
#define __SIMPLE_CONSTRAINTS_H

#include <vector>

#include "nfliteralset.h"

namespace SUBool
{
   /**@brief Encodes an implication into clauses.
    *
    * @param[in] antecedent Literals considered as conjunction
    * @param[in] consequent Literals considered as conjunction
    * @param[out] clauses Where to add the clauses.
    */
   void encode_implication(const std::vector<Literal> &antecedent,
                           const std::vector<Literal> &consequent,
                           std::vector<Clause> &clauses);

   /**@brief Encodes an equivalence into clauses.
    *
    * Equivalence is encoded as two implications.
    *
    * @param[in] left Literals considered as conjunction
    * @param[in] right Literals considered as conjunction
    * @param[out] clauses Where to add the clauses.
    */
   void encode_equivalence(const std::vector<Literal> &left,
                           const std::vector<Literal> &right,
                           std::vector<Clause> &clauses);
} /* namespace SUBool */
#endif
