/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file smutex.cpp
 * Shared mutex (if we cannot use the ones from boost or C++14, C++17).
 */
#include "smutex.h"
#include "sbexception.h"

void
SUBool::SharedMutex::lock()
{
   mExclusive.lock();
}

bool
SUBool::SharedMutex::try_lock()
{
   return mExclusive.try_lock();
}

void
SUBool::SharedMutex::unlock()
{
   mExclusive.unlock();
}

void
SUBool::SharedMutex::lock_shared()
{
   std::lock_guard<std::mutex> shared_guard(mShared);
   ++mReadersCount;
   if (mReadersCount == 1)
   {
      mExclusive.lock();
   }
}

bool
SUBool::SharedMutex::try_lock_shared()
{
   std::lock_guard<std::mutex> shared_guard(mShared);
   if (mReadersCount > 0 || mExclusive.try_lock())
   {
      ++mReadersCount;
      return true;
   }
   return false;
}

void
SUBool::SharedMutex::unlock_shared()
{
   std::lock_guard<std::mutex> shared_guard(mShared);
   if (mReadersCount == 0)
   {
      throw FailureException("SharedMutex::unlock_shared", "no reader");
   }
   if (mReadersCount == 1)
   {
      mExclusive.unlock();
   }
   --mReadersCount;
}
