/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file smutex.h
 * Shared mutex (if we cannot use the one from boost or C++17).
 */

#ifndef __SMUTEX_H
#define __SMUTEX_H

#include <mutex>

namespace SUBool
{
   class SharedMutex
   {
      unsigned mReadersCount{0};
      std::mutex mShared{};
      std::mutex mExclusive{};

    public:
      SharedMutex() = default;
      SharedMutex(const SharedMutex &) = delete;
      ~SharedMutex() = default;
      SharedMutex &operator=(const SharedMutex &) = delete;

      void lock();
      bool try_lock();
      void unlock();

      void lock_shared();
      bool try_lock_shared();
      void unlock_shared();
   };

   class SharedLockGuard
   {
      SharedMutex &mMutex;

    public:
      explicit SharedLockGuard(SharedMutex &m) : mMutex(m)
      {
         mMutex.lock_shared();
      }
      SharedLockGuard(const SharedLockGuard &) = delete;
      ~SharedLockGuard() { mMutex.unlock_shared(); }
   };
} // namespace SUBool

#endif
