/*
 * Project SUBool
 * Petr Kucera, 2020
 */
/**@file types.h
 * Shared basic types
 */

#ifndef __TYPES_H
#define __TYPES_H

#include <algorithm>
#include <cstddef>

namespace SUBool
{
   enum class SetCompareResult
   {
      SUBSET,
      EQUAL,
      SUPERSET,
      INCOMPARABLE
   };

   template <class S> SetCompareResult compare_sets(const S &a, const S &b);
} // namespace SUBool

template <class S>
SUBool::SetCompareResult
SUBool::compare_sets(const S &a, const S &b)
{
   if (a.size() < b.size())
   {
      return std::includes(b.begin(), b.end(), a.begin(), a.end())
                 ? SetCompareResult::SUBSET
                 : SetCompareResult::INCOMPARABLE;
   }
   if (a.size() > b.size())
   {
      return std::includes(a.begin(), a.end(), b.begin(), b.end())
                 ? SetCompareResult::SUPERSET
                 : SetCompareResult::INCOMPARABLE;
   }
   return a == b ? SetCompareResult::EQUAL : SetCompareResult::INCOMPARABLE;
}

#endif
