/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file uccomp.cpp
 * A compiler into a unit refutation complete formula (UC).
 */

#include "uccomp.h"
#include "cnfutil.h"
#include "logstream.h"

const std::string SUBool::UCComp::kEmpowerMsg =
    "Found URC empowering implicate with extending variable";

SUBool::UCComp::UCComp(CNF cnf, Config conf)
    : AbsIncrementalComp(
        std::move(cnf), conf.minimize_method, conf.impl_system_rebuild),
      mUCCheckerSAT(), mUCPCCheckerSAT(), mEncConf(conf.enc_conf),
      mPCEncConf(conf.enc_conf), mUseUC(true), mConf(std::move(conf))
{
   if (mConf.interleave_pc)
   {
      mUCPCCheckerSAT.Init(mEncConf, mPCEncConf, true, mConf.timeouts);
      mUCPCCheckerSAT.SetSolverType(mConf.solver_type);
   }
   else
   {
      mUCCheckerSAT.Init(mEncConf, true, mConf.timeouts);
      mUCCheckerSAT.SetSolverType(mConf.solver_type);
   }
}

const SUBool::AbsComp::BaseConfig &
SUBool::UCComp::GetBaseConfig() const
{
   return mConf;
}

std::unique_ptr<SUBool::AbsEncoder>
SUBool::UCComp::NewPCEncoder(unsigned up_level_bound)
{
   logs.TLog(LogStream::LOG_LEVEL::ALL)
       << "[UCComp::Encoder] Using PCEncoder (p cnf "
       << mHypothesis.MaxVariable() << " " << mHypothesis.Size() << ")"
       << std::endl;
   mUseUC = true;
   mPCEncConf.up_level_bound = up_level_bound;
   return std::unique_ptr<AbsEncoder>(new PCEncoder(mPCEncConf));
}

std::unique_ptr<SUBool::AbsEncoder>
SUBool::UCComp::NewUCEncoder(unsigned up_level_bound)
{
   mUseUC = !mConf.interleave_pc;
   logs.TLog(LogStream::LOG_LEVEL::ALL)
       << "[UCComp::Encoder] Using UCEncoder (p cnf "
       << mHypothesis.MaxVariable() << " " << mHypothesis.Size() << ")"
       << std::endl;
   mEncConf.up_level_bound = up_level_bound;
   return std::unique_ptr<AbsEncoder>(new UCEncoder(mEncConf));
}

const std::string &
SUBool::UCComp::EmpowerMsg()
{
   return kEmpowerMsg;
}

SUBool::AbsCheckerSAT &
SUBool::UCComp::CheckerSAT()
{
   if (mConf.interleave_pc)
   {
      return mUCPCCheckerSAT;
   }
   return mUCCheckerSAT;
}

const SUBool::AbsCheckerSAT &
SUBool::UCComp::CheckerSAT() const
{
   if (mConf.interleave_pc)
   {
      return mUCPCCheckerSAT;
   }
   return mUCCheckerSAT;
}

std::string
SUBool::UCComp::PreambleGoal() const
{
   return "URC";
}

std::string
SUBool::UCComp::PreambleResultFormulaKind() const
{
   return "unit refutation complete";
}
