/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file uccomp.h
 * A compiler into a unit refutation complete formula (UC).
 */

#ifndef __UCCOMP_H
#define __UCCOMP_H

#include "absinccomp.h"
#include "ucenc.h"

namespace SUBool
{
   class UCComp : public AbsIncrementalComp
   {
    public:
      struct Config : BaseConfig
      {
         bool interleave_pc;

         Config() : BaseConfig(), interleave_pc(true) {}

         Config(const BaseConfig &conf) : BaseConfig(conf), interleave_pc(true)
         {
         }

         Config(BaseConfig &&conf)
             : BaseConfig(std::move(conf)), interleave_pc(true)
         {
         }
         Config(const Config &other) = default;
         Config(Config &&other) = default;
         ~Config() = default;
         Config &operator=(const Config &other) = default;
         Config &operator=(Config &&other) = default;
      };
      static const std::string kEmpowerMsg;

    private:
      CheckerSATTemplate<UCEncoder> mUCCheckerSAT;
      UCPCInterleaveCheckerSAT mUCPCCheckerSAT;
      UCEncoder::Config mEncConf;
      PCEncoder::Config mPCEncConf;
      bool mUseUC;
      Config mConf;

      std::unique_ptr<SUBool::AbsEncoder> NewPCEncoder(unsigned up_level_bound);
      std::unique_ptr<SUBool::AbsEncoder> NewUCEncoder(unsigned up_level_bound);

      virtual AbsCheckerSAT &CheckerSAT() override;
      virtual const AbsCheckerSAT &CheckerSAT() const override;
      virtual std::string PreambleGoal() const override;
      virtual std::string PreambleResultFormulaKind() const override;

    public:
      UCComp(CNF cnf, Config conf);
      virtual ~UCComp() = default;
      virtual const BaseConfig &GetBaseConfig() const override;
      virtual const std::string &EmpowerMsg() override;
   };
} // namespace SUBool

#endif
