/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file ucenc.h
 * Encoding of checking if the given CNF admits an empowering
 * implicate with respect to unit refutation completeness.
 */

#include "absenc.h"

#ifndef __UCENC_H
#define __UCENC_H

namespace SUBool
{
   class UCEncoder : public AbsEncoder
   {
    public:
      using Config = AbsConfig;

    private:
      std::vector<unsigned> mPartAssignVars;
      std::vector<Literal> mSelectLits;

      Config mConf;

      void Rebuild();

      virtual void WriteMappingsOfVariable(
          std::ostream &out, unsigned i) const override;
      virtual std::string Name() const override;

      void PartAssignClosedUnderUP(
          const CNF &cnf, const ClausesToLiterals &cl2lit);
      void PartAssignNoConflict();
      void InitSelectLits();
      void SelectedNotInPA();
      void OneProve(bool positive);
      void PassInputsToOneProve(bool positive, const std::vector<unsigned> &in);
      void ConflictInOneProve(const std::vector<unsigned> &out);
      virtual void AddClauses(const CNF &cnf, const ClausesToLiterals &cl2lit,
          std::shared_ptr<const EncISProvider> impl_system_provider) override;

    protected:
      virtual const AbsConfig &GetConfig() const override;

    public:
      UCEncoder(const Config &conf = Config());

      // Returns the variable over which the parent clauses resolve
      virtual unsigned ExtractClause(
          const SolverInterface &solver, Clause *clause) const override;
   };
} // namespace SUBool

#endif
