/*
 * Project SUBool
 * Petr Kucera, 2020
 */
/**@file up_impl_level.h
 * Encoding of unit propagation based on assigning k-bit levels to each variable
 * and implication.
 */
#ifndef __UP_IMPL_LEVEL_H
#define __UP_IMPL_LEVEL_H

#include "impl_system.h"
#include "up_levels.h"
#include "upenc.h"

namespace SUBool
{
   class UPEncImplLevel : public UPEncoder
   {

    public:
      struct Config
      {
         bool use_bit_level{true};

         Config() {}

         Config(bool ubl) : use_bit_level(ubl) {}

         void Dump(const std::string &line_prefix, unsigned level) const;
      };

      struct impl_var_t
      {
         unsigned aux_var{0};     // in the encoding
         Literal level_literal{}; // in the level_vars
      };

    private:
      const LitImplSystem &mImplSystem;
      Config mConf;

      unsigned BitSizeForLevel(unsigned level) const;
      void AddUnitsForEmptyBodies(const std::vector<unsigned> inputs,
          std::vector<Clause> *clauses) const;
      void AddImplication(unsigned impl_index,
          // const std::vector<unsigned> &inputs,
          const std::vector<unsigned> &outputs, UPLevels &level_vars,
          std::vector<Clause> &lit_clauses, bool &new_var,
          std::vector<Clause> *clauses) const;
      std::vector<Clause> InitLitClauses(const std::vector<unsigned> &inputs,
          const std::vector<unsigned> &outputs) const;
      void AssumptionsImplyConsequences(const std::vector<unsigned> &inputs,
          const std::vector<unsigned> &outputs,
          std::vector<Clause> *clauses) const;
      void AssumptionsHaveZeroLevel(const std::vector<unsigned> &inputs,
          const UPLevels &level_vars, std::vector<Clause> *clauses) const;
      void AddImplToLitClause(const Literal &cons_lit, unsigned impl_index,
          impl_var_t impl_var, UPLevels &level_vars, Clause &lit_clause,
          std::vector<Clause> *clauses) const;
      impl_var_t InitImplVar(unsigned impl_index,
          const std::vector<unsigned> &outputs, UPLevels &level_vars,
          bool &new_var) const;
      void AddBodyLitClauses(impl_var_t impl_var, const Literal &body_literal,
          UPLevels &level_vars, const std::vector<unsigned> &outputs,
          std::vector<Clause> *clauses) const;
      NFSize CountAddUnitsForEmptyBodies() const;
      NFSize CountAddImplication(
          unsigned impl_index, unsigned bit_size, bool &new_var) const;
      NFSize CountAssumptionsImplyConsequences() const;
      NFSize CountAssumptionsHaveZeroLevel(unsigned bit_size) const;
      NFSize CountInitImplVar(
          unsigned impl_index, unsigned bit_size, bool &new_var) const;

    public:
      UPEncImplLevel(const Config &conf, const CNF &cnf,
          const ClausesToLiterals &cl2lit, VarStore &var_store,
          const LitImplSystem &mImplSystem);
      virtual void Encode(unsigned max_level, std::vector<unsigned> *inputs,
          std::vector<unsigned> *outputs,
          std::vector<Clause> *clauses) const override;
      virtual NFSize EstimateSize(unsigned max_level) const override;
      virtual unsigned MaxLevelBound() const override;
      virtual unsigned NextLevelBound(unsigned level) const override;
   };
} // namespace SUBool

inline unsigned
SUBool::UPEncImplLevel::BitSizeForLevel(unsigned level) const
{
   return bit_size_for_level(level, mCnf.MaxVariable());
}

#endif

