/*
 * Project SUBool
 * Petr Kucera, 2017
 */
/**@file up_levels.h
 * Functions for encoding levels of unit propagations encoded in
 * binary and their comparisons.
 */

#ifndef __UP_LEVELS_H
#define __UP_LEVELS_H

#include "normalform.h"
#include "varstore.h"

namespace SUBool
{

   unsigned new_aux_comparison_var(const std::string &name_base,
       const std::string &left_desc, const std::string &right_desc,
       unsigned bit, VarStore &var_store);
   unsigned encode_bitvector_gt(const std::vector<unsigned> &left_bv,
       const std::vector<unsigned> &right_bv, VarStore &var_store,
       std::vector<Clause> *clauses, const std::string &name_base,
       const std::string &left_desc, const std::string &right_desc);

   class UPLevels
   {
      const CNF &mCnf;
      VarStore &mVarStore;
      bool mLiteralLevels;
      unsigned mLevelBound;
      std::vector<std::vector<unsigned>> mLevelVars;
      const std::string mNameBase;
      unsigned mMaxVariable;

      void AddLiteral(const Literal &lit);

    public:
      UPLevels(const CNF &cnf, VarStore &var_store, bool literal_levels,
          unsigned level_bound, const std::string &name_base);

      const std::vector<unsigned> &LevelVars(unsigned var) const;
      const std::vector<unsigned> &LevelVars(const Literal &lit) const;
      void AddVariable();
      unsigned EncodeGT(const Literal &left, const Literal &right,
          std::vector<Clause> *clauses) const;
      unsigned MaxVariable() const;
      unsigned
      LevelBound() const
      {
         return mLevelBound;
      }
      unsigned
      BitSize() const
      {
         return mLevelBound;
      }

      static NFSize CountInitialSize(
          const CNF &cnf, bool literal_levels, unsigned level_bound);
      static NFSize CountAddVariable(bool literal_levels, unsigned level_bound);
      static NFSize CountEncodeGTSize(
          bool literal_levels, unsigned level_bound);
   };
}; // namespace SUBool

#endif
