/*
 * Project SUBool
 * Petr Kucera, 2017
 */
/**@file up_max.h
 * Encoding of unit propagation constraint based on selecting maximum level of a
 * literal.
 */

#ifndef __UP_MAX_H
#define __UP_MAX_H

#include "up_levels.h"
#include "upenc.h"

namespace SUBool
{
   class UPEncMaxLevel : public UPEncoder
   {

      void AddClause(const Clause &clause, unsigned index,
                     const UPLevels &level_vars,
                     std::vector<std::vector<Literal>> &fire_clauses,
                     const std::vector<unsigned> &inputs,
                     const std::vector<unsigned> &outputs,
                     std::vector<Clause> *clauses) const;
      void AddUnitClause(const Clause &clause, unsigned index,
                         const UPLevels &level_vars,
                         std::vector<std::vector<Literal>> &fire_clauses,
                         const std::vector<unsigned> &inputs,
                         const std::vector<unsigned> &outputs,
                         std::vector<Clause> *clauses) const;
      void AddBinaryClause(const Clause &clause, unsigned index,
                           const UPLevels &level_vars,
                           std::vector<std::vector<Literal>> &fire_clauses,
                           const std::vector<unsigned> &inputs,
                           const std::vector<unsigned> &outputs,
                           std::vector<Clause> *clauses) const;
      void AddBigClause(const Clause &clause, unsigned index,
                        const UPLevels &level_vars,
                        std::vector<std::vector<Literal>> &fire_clauses,
                        const std::vector<unsigned> &inputs,
                        const std::vector<unsigned> &outputs,
                        std::vector<Clause> *clauses) const;
      void AddFireClauses(std::vector<std::vector<Literal>> &fire_clauses,
                          const std::vector<unsigned> &inputs,
                          const std::vector<unsigned> &outputs,
                          std::vector<Clause> *clauses) const;
      void InitClauseVars(unsigned max_level, std::vector<unsigned> *vars,
                          unsigned c_index) const;
      void InitClauseIntermedVars(unsigned max_level,
                                  std::vector<unsigned> *vars, unsigned c_index,
                                  unsigned var) const;

      unsigned NewFireVar(unsigned c_index, const Literal &lit) const;

    public:
      UPEncMaxLevel(const CNF &cnf, const ClausesToLiterals &cl2lit,
                    VarStore &var_store);
      virtual void Encode(unsigned max_level, std::vector<unsigned> *inputs,
                          std::vector<unsigned> *outputs,
                          std::vector<Clause> *clauses) const override;
      virtual NFSize EstimateSize(unsigned max_level) const override;
      virtual unsigned MaxLevelBound() const override;
      virtual unsigned NextLevelBound(unsigned level) const override;
   };
}; // namespace SUBool

#endif
