/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file up_min_lq.h
 * An encoding of unit propagation constraint. It chooses either quadratic or
 * logarithmic encoding based on their sizes.
 */

#ifndef __UP_MIN_LQ_H
#define __UP_MIN_LQ_H

#include "up_log.h"
#include "up_quad.h"
#include "upenc.h"

namespace SUBool
{
   class UPEncMinLQ : public UPEncoder
   {
    public:
      struct SizeWeights
      {
         double max_variable{1.0};
         double size{0.0};
         double len{0.0};

         SizeWeights(){};
         SizeWeights(double mv, double s, double l)
             : max_variable(mv), size(s), len(l){};

         friend std::ostream &
         operator<<(std::ostream &output, const SizeWeights &w)
         {
            return output << "{.max_variable=" << w.max_variable
                          << ", .size=" << w.size << ", .len=" << w.len << "}";
         }
      };

    private:
      SizeWeights mLogWeights;
      SizeWeights mQuadWeights;

      UPEncQuad mEncQuad;
      UPEncLogarithmic mEncLog;

      std::pair<NFSize, double> WeightedSize(const UPEncoder &enc,
                                             unsigned level,
                                             const SizeWeights &weights) const;
      bool PickQuad(unsigned level, NFSize *size = nullptr) const;

    public:
      UPEncMinLQ(const CNF &cnf, const ClausesToLiterals &cl2lit,
                 VarStore &var_store, const UPEncLogarithmic::Config &log_conf,
                 SizeWeights log_weights = SizeWeights(),
                 SizeWeights quad_weights = SizeWeights());
      virtual void Encode(unsigned max_level, std::vector<unsigned> *inputs,
                          std::vector<unsigned> *outputs,
                          std::vector<Clause> *clauses) const override;
      virtual NFSize EstimateSize(unsigned max_level) const override;
      virtual unsigned MaxLevelBound() const override;
      virtual unsigned NextLevelBound(unsigned level) const override;
   };
} // namespace SUBool

#endif
