/*
 * Project SUBool
 * Petr Kucera, 2017
 */
/**@file up_onelevel.cpp
 * One level encoding of unit propagation constraint. Only suitable
 * for one implication.
 */

#include "up_onelevel.h"

void
SUBool::UPEncOneLevel::InitVars(std::vector<unsigned> *vars) const
{
   vars->resize(2 * mCnf.MaxVariable());
   for (unsigned i = 0; i < 2 * mCnf.MaxVariable(); ++i)
   {
      mVarStore.NameStream() << "_{" << Literal(i) << "}";
      (*vars)[i] = mVarStore.NewVariable();
   }
}

void
SUBool::UPEncOneLevel::Encode(unsigned /*max_level*/,
                              std::vector<unsigned> *inputs,
                              std::vector<unsigned> *outputs,
                              std::vector<Clause> *clauses) const
{
   if (inputs == nullptr)
   {
      throw NullPointerException("UPEncOneLevel::Encode", "inputs == nullptr");
   }
   if (clauses == nullptr)
   {
      throw NullPointerException("UPEncOneLevel::Encode", "clauses == nullptr");
   }
   if (outputs != nullptr)
   {
      outputs->clear();
   }
   InitVars(inputs);
   for (const auto &clause : mCnf)
   {
      for (const auto &lit : clause)
      {
         std::vector<Literal> clause_lit;
         for (const auto &f : clause)
         {
            if (f == lit)
            {
               clause_lit.emplace_back((*inputs)[f.Index()], true);
            }
            else
            {
               clause_lit.emplace_back((*inputs)[f.Negation().Index()], false);
            }
         }
         clauses->emplace_back(clause_lit);
      }
   }
}

SUBool::NFSize
SUBool::UPEncOneLevel::EstimateSize(unsigned /*max_level*/) const
{
   NFSize s{0, 0, 0};
   s.max_variable = 2 * mCnf.MaxVariable();
   for (const auto &clause : mCnf)
   {
      s.size += clause.Size();
      s.len += clause.Size() * clause.Size();
   }
   return s;
}

unsigned
SUBool::UPEncOneLevel::MaxLevelBound() const
{
   return 1;
}

unsigned
SUBool::UPEncOneLevel::NextLevelBound(unsigned) const
{
   return 0;
}
