/*
 * Project SUBool
 * Petr Kucera, 2021
 */
/**@file up_onelevel_is.cpp
 * One level encoding of unit propagation constraint based on implication
 * system. Only suitable for checking empowerement.
 */

#include "up_onelevel_is.h"

void
SUBool::UPEncOneLevelIS::InitVars(std::vector<unsigned> *vars) const
{
   vars->resize(2 * mCnf.MaxVariable());
   for (unsigned i = 0; i < 2 * mCnf.MaxVariable(); ++i)
   {
      mVarStore.NameStream() << "_{" << Literal(i) << "}";
      (*vars)[i] = mVarStore.NewVariable();
   }
}

void
SUBool::UPEncOneLevelIS::Encode(unsigned /*max_level*/,
    std::vector<unsigned> *inputs, std::vector<unsigned> *outputs,
    std::vector<Clause> *clauses) const
{
   if (inputs == nullptr)
   {
      throw NullPointerException(
          "UPEncOneLevelIS::Encode", "inputs == nullptr");
   }
   if (clauses == nullptr)
   {
      throw NullPointerException(
          "UPEncOneLevelIS::Encode", "clauses == nullptr");
   }
   if (outputs != nullptr)
   {
      outputs->clear();
   }
   InitVars(inputs);
   std::vector<Literal> clause_buffer;
   for (const auto &impl : mImplSystem)
   {
      clause_buffer.clear();
      for (const auto &body_lit : impl.Body())
      {
         clause_buffer.emplace_back(inputs->at(body_lit.Index()), false);
      }
      for (const auto &cons_lit : impl.Cons())
      {
         clause_buffer.emplace_back(inputs->at(cons_lit.Index()), true);
         clauses->emplace_back(clause_buffer);
         clause_buffer.pop_back();
      }
   }
}

SUBool::NFSize
SUBool::UPEncOneLevelIS::EstimateSize(unsigned /*max_level*/) const
{
   NFSize s{0, 0, 0};
   s.max_variable = 2 * mCnf.MaxVariable();
   for (const auto &impl : mImplSystem)
   {
      s.size += impl.ConsSize();
      s.len += (impl.BodySize() + 1) * impl.ConsSize();
   }
   return s;
}

unsigned
SUBool::UPEncOneLevelIS::MaxLevelBound() const
{
   return 1;
}

unsigned
SUBool::UPEncOneLevelIS::NextLevelBound(unsigned) const
{
   return 0;
}
