/*
 * Project SUBool
 * Petr Kucera, 2021
 */
/**@file up_onelevel_is.h
 * One level encoding of unit propagation constraint based on implication
 * system. Only suitable for checking empowerement.
 */

#ifndef __UP_ONELEVEL_IS_H
#define __UP_ONELEVEL_IS_H

#include "impl_system.h"
#include "upenc.h"

namespace SUBool
{
   class UPEncOneLevelIS : public UPEncoder
   {
    private:
      LitImplSystem mImplSystem;

      void InitVars(std::vector<unsigned> *vars) const;

    public:
      UPEncOneLevelIS(const CNF &cnf, const ClausesToLiterals &cl2lit,
          VarStore &var_store, LitImplSystem is)
          : UPEncoder(cnf, cl2lit, var_store), mImplSystem(std::move(is))
      {
      }
      virtual void Encode(unsigned max_level, std::vector<unsigned> *inputs,
          std::vector<unsigned> *outputs,
          std::vector<Clause> *clauses) const override;
      virtual NFSize EstimateSize(unsigned max_level) const override;
      virtual unsigned MaxLevelBound() const override;
      virtual unsigned NextLevelBound(unsigned level) const override;
   };
}; // namespace SUBool

#endif
