/*
 * Project SUBool
 * Petr Kucera, 2017
 */
/**@file up_quad.h
 * Quadratic size encoding of unit propagation constraint. Allows the
 * two implications separately or together.
 */

#ifndef __UP_QUAD_H
#define __UP_QUAD_H

#include "upenc.h"

namespace SUBool
{
   class UPEncQuad : public UPEncoder
   {
    public:
      enum DIRECTION
      {
         IF,
         ONLY_IF,
         BOTH
      };

    private:
      DIRECTION mDirection;

      void InitVars(std::vector<std::vector<unsigned>> *vars,
                    unsigned max_level) const;

      void AddClauses(const std::vector<std::vector<unsigned>> &vars,
                      std::vector<Clause> *clauses) const;
      void AddClausesAtLevel(const std::vector<unsigned> &prev_vars,
                             const std::vector<unsigned> &vars, unsigned level,
                             std::vector<Clause> *clauses) const;
      void AddClausesForLiteralAtLevel(const Literal &lit,
                                       const unsigned var_out,
                                       const std::vector<unsigned> &prev_vars,
                                       unsigned level,
                                       std::vector<Clause> *clauses) const;
      void AddClausesForLiteralAtLevelIF(const Literal &lit,
                                         const unsigned var_out,
                                         const std::vector<unsigned> &prev_vars,
                                         unsigned level,
                                         std::vector<Clause> *clauses) const;
      void AddClausesForLiteralAtLevelONLYIF(
          const Literal &lit, const unsigned var_out,
          const std::vector<unsigned> &prev_vars, unsigned level,
          std::vector<Clause> *clauses) const;

      NFSize CountInitVars(unsigned max_level) const;
      NFSize CountAddClauses(unsigned max_level, NFSize s) const;
      NFSize CountAddClausesAtLevel() const;
      NFSize CountAddClausesForLiteralAtLevel(const Literal &lit) const;
      NFSize CountAddClausesForLiteralAtLevelIF(const Literal &lit) const;
      NFSize CountAddClausesForLiteralAtLevelONLYIF(const Literal &lit) const;

    public:
      UPEncQuad(DIRECTION direction, const CNF &cnf,
                const ClausesToLiterals &cl2lit, VarStore &var_store);
      virtual void Encode(unsigned max_level, std::vector<unsigned> *inputs,
                          std::vector<unsigned> *outputs,
                          std::vector<Clause> *clauses) const override;
      virtual NFSize EstimateSize(unsigned max_level) const override;
      virtual unsigned MaxLevelBound() const override;
      virtual unsigned NextLevelBound(unsigned level) const override;
   };
}; // namespace SUBool

#endif
