/*
 * Project SUBool
 * Petr Kucera, 2020
 */
/**@file up_quad_impl.h
 * Quadratic size encoding of unit propagation constraint which uses a given
 * implication system equivalent to the dual rail encoding of the CNF to be
 * encoded.
 */
#ifndef __UP_QUAD_IMPL_H
#define __UP_QUAD_IMPL_H

#include "impl_system.h"
#include "upenc.h"

namespace SUBool
{
   class UPEncQuadImpl : public UPEncoder
   {
      const LitImplSystem &mImplSystem;

      void InitVars(
          std::vector<std::vector<unsigned>> *vars, unsigned max_level) const;
      void AddClauses(const std::vector<std::vector<unsigned>> &vars,
          std::vector<Clause> *clauses) const;
      void AddClausesAtLevel(const std::vector<unsigned> &prev_vars,
          const std::vector<unsigned> &vars, unsigned level,
          std::vector<Clause> *clauses) const;
      std::vector<Clause> InitLitClauses(const std::vector<unsigned> &prev_vars,
          const std::vector<unsigned> &vars) const;
      void AddUnitsForEmptyBodies(const std::vector<unsigned> input_vars,
          std::vector<Clause> *clauses) const;
      void AddClausesForImplAtLevel(const LitImpl &impl, size_t impl_index,
          const std::vector<unsigned> &prev_vars, unsigned level,
          std::vector<Clause> *lit_clauses, std::vector<Clause> *clauses) const;
      Literal BodyLiteral(const LitImpl &impl, size_t impl_index,
          const std::vector<unsigned> &prev_vars, unsigned level) const;
      NFSize CountInitVars(unsigned max_level) const;
      NFSize CountAddClauses(unsigned max_level, NFSize s) const;
      NFSize CountAddClausesAtLevel() const;
      NFSize CountAddUnitsForEmptyBodies() const;

    public:
      UPEncQuadImpl(const CNF &cnf, const ClausesToLiterals &cl2lit,
          VarStore &var_store, const LitImplSystem &is);

      virtual void Encode(unsigned max_level, std::vector<unsigned> *inputs,
          std::vector<unsigned> *outputs,
          std::vector<Clause> *clauses) const override;
      virtual NFSize EstimateSize(unsigned max_level) const override;
      virtual unsigned MaxLevelBound() const override;
      virtual unsigned NextLevelBound(unsigned level) const override;
   };
} // namespace SUBool

#endif /* end of include guard */
