/*
 * Project SUBool
 * Petr Kucera, 2017
 */
/**@file upenc.h
 * Encoding of unit propagation constraint. This declares the base
 * class.
 */

#ifndef __UPENC_H
#define __UPENC_H

#include <vector>

#include "cl2lit.h"
#include "normalform.h"
#include "varstore.h"

namespace SUBool
{
   class UPEncoder
   {
    protected:
      const CNF &mCnf;
      const ClausesToLiterals &mCl2Lit;
      VarStore &mVarStore;

    public:
      UPEncoder(
          const CNF &cnf, const ClausesToLiterals &cl2lit, VarStore &var_store)
          : mCnf(cnf), mCl2Lit(cl2lit), mVarStore(var_store)
      {
      }
      virtual ~UPEncoder() = default;

      virtual void Encode(unsigned max_level, std::vector<unsigned> *inputs,
          std::vector<unsigned> *outputs,
          std::vector<Clause> *clauses) const = 0;

      // (max_variable, size)
      virtual NFSize EstimateSize(unsigned max_level) const = 0;
      virtual unsigned MaxLevelBound() const = 0;
      // Returns 0 if there is no level beyond the given one
      virtual unsigned NextLevelBound(unsigned level) const = 0;
   };

   inline unsigned
   bit_size_for_level(unsigned level, unsigned max_level)
   {
      if (level == 0 || level > max_level)
      {
         level = max_level;
      }
      return static_cast<unsigned>(ceil(log2(static_cast<double>(level + 1))));
   }

}; // namespace SUBool

#endif
