/*
 * Project SUBool
 * Petr Kucera, 2017
 */
/**@file varstore.h
 * Class for storing and creating new variables.
 */

#ifndef __VARSTORE_H
#define __VARSTORE_H

#include "nfliteralset.h"
#include "nullstream.h"
#include <sstream>
#include <vector>

namespace SUBool
{
   class VarStore
   {
      NullBuffer mNullBuffer;

      bool mUseBackMap{false};
      unsigned mLastVariable{0};
      std::vector<std::string> mNames;
      std::string mEmptyName;
      std::stringstream mNameStream;
      std::string mPrefix;

    public:
      VarStore(bool use_backmap, unsigned last_variable = 0);

      VarStore(const VarStore &);
      VarStore(VarStore &&) noexcept;
      VarStore &operator=(const VarStore &);
      VarStore &operator=(VarStore &&) noexcept;

      void Clear(unsigned last_variable);

      unsigned NewVariable(const std::string &name);
      unsigned NewVariable(); // The name is previously given via NameStream
      void InitVarArray(std::vector<unsigned> *vars, unsigned n, bool dual_rail,
                        const std::string &prefix, const std::string &suffix);
      void InitLitArray(std::vector<Literal> *lits, unsigned n, bool dual_rail,
                        const std::string &prefix, const std::string &suffix);

      std::vector<unsigned> VarArray(unsigned n, bool dual_rail,
                                     const std::string &prefix,
                                     const std::string &suffix);
      std::vector<Literal> LitArray(unsigned n, bool dual_rail,
                                    const std::string &prefix,
                                    const std::string &suffix);
      const std::string &Name(unsigned var) const;
      unsigned LastVariable() const;
      void WriteClauseComment(std::ostream &out, const Clause &clause) const;
      void
      SetPrefix(const std::string &prefix)
      {
         mPrefix = prefix;
      }

      std::ostream &NameStream();
   };
}; // namespace SUBool

#endif
