#!/bin/sh

# Project SUBool
# Petr Kucera, 2018
#
# Script for exporting the sources to a tgz archive.

scriptname="$0"

usage() {
   echo 'Project SUBool'
   echo 'export -- script for exporting the source files into a tar archive.'
   echo 'Petr Kucera, 2018'
   echo
   printf 'Usage: %s [-q|-h] [output file]\n\n' "$scriptname"
   echo "Uses variable SUBOOL_TOP to determine the top directory "\
      "of the SUBool project. If this variable is empty, the script "\
      "attempts to determine the top directory. If this fails, you "\
      "can try to set the value of SUBOOL_TOP to the correct value "\
      "before calling the script."
   echo
   echo "   -h - print the help screen and finish"
   echo "   -q - do not write anything to the output, exit code still "\
      "determines if export was successful "
}

quiet=false
gvopt=
while getopts hq optname
do
   case $optname in
      h) usage ; exit 1 ;;
      q) quiet=true ; gvopt=-q ;;
      *) usage ; exit 1 ;;
   esac
done
shift $((OPTIND-1))

target=$(pwd)
if [ "$#" -eq 1 ]
then
   targ_dir=$(dirname "$1")
   target=$(cd "$targ_dir" && pwd)/$(basename "$1")
fi
if [ -d "$target" ]
then
   target="$target/subool.tgz"
fi

if [ -z "$SUBOOL_TOP" ]
then
   mydir=$(dirname "$0")
   SUBOOL_TOP=$(cd "$mydir" && cd .. && pwd)
fi

cd "$SUBOOL_TOP" || { echo "Failed to determine SUBool top directory."; exit 1; }
if ! script/genver.sh "$gvopt"
then
   "$quiet" || echo "WARNING: Failed to update version info."
fi

above=$(basename "$SUBOOL_TOP")
cd ..

tar czf "$target" "$above/bin" "$above/lib" "$above/script"\
   "$above/test" "$above/CMakeLists.txt" "$above/Makefile"

