/*
 * Project SUBool
 * Petr Kucera, 2016
 */
/**@file test_assign.cpp
 * Contains tests for AssignmentTemplate and derived classes.
 */

#include "test_assign.h"

bool
SUBool::TestAssign::TestPartialValue() const
{
   std::cout << "Checking partial value logic. ";
   std::vector<std::vector<PartialValue>> l_not = {
       {PartialValue::FALSE, PartialValue::TRUE},
       {PartialValue::TRUE, PartialValue::FALSE},
       {PartialValue::UNDEF, PartialValue::UNDEF},
   };
   std::vector<std::vector<PartialValue>> l_and = {
       {PartialValue::TRUE, PartialValue::TRUE, PartialValue::TRUE},
       {PartialValue::TRUE, PartialValue::FALSE, PartialValue::FALSE},
       {PartialValue::TRUE, PartialValue::UNDEF, PartialValue::UNDEF},
       {PartialValue::FALSE, PartialValue::TRUE, PartialValue::FALSE},
       {PartialValue::FALSE, PartialValue::FALSE, PartialValue::FALSE},
       {PartialValue::FALSE, PartialValue::UNDEF, PartialValue::FALSE},
       {PartialValue::UNDEF, PartialValue::TRUE, PartialValue::UNDEF},
       {PartialValue::UNDEF, PartialValue::FALSE, PartialValue::FALSE},
       {PartialValue::UNDEF, PartialValue::UNDEF, PartialValue::UNDEF}};
   std::vector<std::vector<PartialValue>> l_or = {
       {PartialValue::TRUE, PartialValue::TRUE, PartialValue::TRUE},
       {PartialValue::TRUE, PartialValue::FALSE, PartialValue::TRUE},
       {PartialValue::TRUE, PartialValue::UNDEF, PartialValue::TRUE},
       {PartialValue::FALSE, PartialValue::TRUE, PartialValue::TRUE},
       {PartialValue::FALSE, PartialValue::FALSE, PartialValue::FALSE},
       {PartialValue::FALSE, PartialValue::UNDEF, PartialValue::UNDEF},
       {PartialValue::UNDEF, PartialValue::TRUE, PartialValue::TRUE},
       {PartialValue::UNDEF, PartialValue::FALSE, PartialValue::UNDEF},
       {PartialValue::UNDEF, PartialValue::UNDEF, PartialValue::UNDEF}};

   for (const auto &neg : l_not)
   {
      if ((!neg[0]) != neg[1])
      {
         std::cout << "Mismatch !" << neg[0] << "!=" << neg[1] << " [FAILED]"
                   << std::endl;
         return false;
      }
   }
   for (const auto &val : l_and)
   {
      if ((val[0] & val[1]) != val[2])
      {
         std::cout << "Mismatch " << val[0] << " && " << val[1]
                   << "!=" << val[2] << " [FAILED]" << std::endl;
         return false;
      }
   }
   for (const auto &val : l_or)
   {
      if ((val[0] | val[1]) != val[2])
      {
         std::cout << "Mismatch " << val[0] << " || " << val[1]
                   << "!=" << val[2] << " [FAILED]" << std::endl;
         return false;
      }
   }
   std::cout << " [OK]" << std::endl;
   return true;
}

bool
SUBool::TestAssign::TestFalsePartial(unsigned n) const
{
   std::cout << "Test false partial assignment (size " << n << "): ";
   PartialAssignment alpha(n, PartialValue::FALSE);
   std::cout << alpha << std::endl;
   bool size_ok = (alpha.Size() == n);
   std::cout << "Size: " << alpha.Size() << std::endl
             << " " << (size_ok ? "[PASSED]" : "[FAILED]") << std::endl;
   bool values_ok = true;
   for (const auto &val : alpha)
   {
      if (val != PartialValue::FALSE)
      {
         values_ok = false;
         break;
      }
   }
   std::cout << "Values should be FALSE "
             << (values_ok ? "[PASSED]" : "[FAILED]") << std::endl;
   return values_ok && size_ok;
}

bool
SUBool::TestAssign::TestFalseFull(unsigned n) const
{
   std::cout << "Test false assignment (size " << n << "): ";
   Assignment alpha(n, false);
   std::cout << alpha << std::endl;
   bool size_ok = (alpha.Size() == n);
   std::cout << "Size: " << alpha.Size() << std::endl
             << " " << (size_ok ? "[PASSED]" : "[FAILED]") << std::endl;
   bool values_ok = true;
   for (const auto val : alpha)
   {
      if (val)
      {
         values_ok = false;
         break;
      }
   }
   std::cout << "Values should be false "
             << (values_ok ? "[PASSED]" : "[FAILED]") << std::endl;
   return values_ok && size_ok;
}

bool
SUBool::TestAssign::TestRandomPartial(unsigned n, double p0, double p1) const
{
   std::cout << "Test random partial assignment (size " << n << "): ";
   PartialAssignment alpha(n, p0, p1);
   std::cout << alpha << std::endl;
   bool size_ok = (alpha.Size() == n);
   std::cout << "Size: " << alpha.Size() << std::endl
             << " " << (size_ok ? "[PASSED]" : "[FAILED]") << std::endl;
   return size_ok;
}

bool
SUBool::TestAssign::TestRandomFull(unsigned n, double p1) const
{
   std::cout << "Test random assignment (size " << n << "): ";
   Assignment alpha(n, p1);
   std::cout << alpha << std::endl;
   bool size_ok = (alpha.Size() == n);
   std::cout << "Size: " << alpha.Size() << std::endl
             << " " << (size_ok ? "[PASSED]" : "[FAILED]") << std::endl;
   return size_ok;
}

bool
SUBool::TestAssign::TestIncFull() const
{
   int cnt = 0;
   std::cout << "Testing full assignment increase." << std::endl;
   Assignment alpha(4);
   alpha.Reset();
   do
   {
      std::cout << alpha << std::endl;
      ++cnt;
   } while (alpha.Inc());
   if (cnt == 16)
   {
      std::cout << "[OK]" << std::endl;
      return true;
   }
   std::cout << "[FAILED]" << std::endl;
   return true;
}

bool
SUBool::TestAssign::TestIncPartial() const
{
   int cnt = 0;
   std::cout << "Testing partial assignment increase." << std::endl;
   PartialAssignment alpha(4);
   alpha.Reset();
   do
   {
      std::cout << alpha << std::endl;
      ++cnt;
   } while (alpha.Inc());
   if (cnt == 81)
   {
      std::cout << "[OK]" << std::endl;
      return true;
   }
   std::cout << "[FAILED]" << std::endl;
   return true;
}

bool
SUBool::TestAssign::RunTests() const
{
   bool success = TestPartialValue() && TestFalsePartial(10)
                  && TestFalseFull(10) && TestRandomPartial(10, .333, .333)
                  && TestRandomFull(10, .5) && TestIncFull()
                  && TestIncPartial();
   return success;
}

int
main(int /*argc*/, char * /*argv*/[])
{
   srand(time(NULL));
   SUBool::TestAssign test;
   return !test.RunTests();
}
