/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file test_cnfcompress.cpp
 * Contains tests for CNFCompress.
 */

#include <cmath>
#include <iostream>
#include <sstream>

#include "dimacs.h"
#include "test_cnfcompress.h"

const char *dimacs = "c\n"
                     "p cnf 16 7\n"
                     "1 3 5 9 0\n"
                     "-1 4 7 -16 0\n"
                     "-3 -5 11 0\n"
                     "-4 -7 -11 16 0\n"
                     "-2 7 16 0\n"
                     "1 3 4 -9 0\n"
                     "11 -16 0\n";

const char *dimacs_odd = "c\n"
                         "p cnf 21 7\n"
                         "1 3 5 9 0\n"
                         "-1 -7 11 13 0\n"
                         "15 19 21 0\n"
                         "3 5 -7 0\n"
                         "7 13 -19 -21 0\n"
                         "3 11 -13 0\n"
                         "1 11 13 -15 17 0";

bool
SUBool::TestCNFCompress::TestCompress() const
{
   std::cout << "Testing CNFCompress." << std::endl;
   auto cnf = Dimacs::parse_cnf(dimacs);
   std::cout << "Input CNF:" << std::endl;
   std::cout << cnf << std::endl;
   CNFCompressor compress(cnf);
   auto c_cnf = compress.CompressCNF(cnf);
   std::cout << "Compressed CNF:" << std::endl;
   std::cout << c_cnf << std::endl;
   CNF uncomp = compress.UncompressCNF(c_cnf);
   std::cout << "Uncompressed CNF:" << std::endl;
   std::cout << uncomp << std::endl;
   return (cnf == uncomp);
}

bool
SUBool::TestCNFCompress::TestCompressWithFixedVariables() const
{
   std::cout << "Testing CNFCompress with max_fixed_variable=8." << std::endl;
   auto cnf = Dimacs::parse_cnf(dimacs_odd);
   std::cout << "Input CNF:" << std::endl;
   std::cout << cnf << std::endl;
   CNFCompressor compress(cnf, 8);
   auto c_cnf = compress.CompressCNF(cnf);
   std::cout << "Compressed CNF:" << std::endl;
   std::cout << c_cnf << std::endl;
   CNF uncomp = compress.UncompressCNF(c_cnf);
   std::cout << "Uncompressed CNF:" << std::endl;
   std::cout << uncomp << std::endl;
   return (cnf == uncomp);
}

bool
SUBool::TestCNFCompress::RunTests() const
{
   return TestCompress() && TestCompressWithFixedVariables();
}

int
main(int /*argc*/, char * /*argv*/[])
{
   srand(time(NULL));
   SUBool::TestCNFCompress test;
   return !test.RunTests();
}
