/*
 * Project SUBool
 * Petr Kucera, 2019
 */
/**@file test_common.cpp
 * Contains common functions used in other tests.
 */

#include "test_common.h"

bool
SUBool::test_encoding_consistency_full(const CNFEncoding &enc,
                                       const CNFEncoding &ref_enc)
{
   std::cout << "Checking consistency of an encoding with reference encoding "
                "on full assignments";
   Assignment alpha(ref_enc.MaxInputVariable());
   do
   {
      if (enc.Consistent(alpha) != ref_enc.Consistent(alpha))
      {
         std::cout << " [FAILED]" << std::endl;
         return false;
      }
   } while (alpha.Inc());
   std::cout << " [OK]" << std::endl;
   return true;
}

bool
SUBool::test_encoding_consistency_partial(const CNFEncoding &enc,
                                          const CNFEncoding &ref_enc)
{
   std::cout << "Checking consistency of an encoding with reference encoding "
                "on partial assignments";
   PartialAssignment alpha(ref_enc.MaxInputVariable());
   do
   {
      if (enc.Consistent(alpha) != ref_enc.Consistent(alpha))
      {
         std::cout << " [FAILED]" << std::endl;
         return false;
      }
   } while (alpha.Inc());
   std::cout << " [OK]" << std::endl;
   return true;
}

bool
SUBool::test_encoding_propagation(const CNFEncoding &enc,
                                  const CNFEncoding &ref_enc)
{
   std::cout << "Checking propagation of an encoding with reference encoding "
                "on partial assignments";
   PartialAssignment alpha(ref_enc.MaxInputVariable());
   do
   {
      if (enc.Propagate(alpha) != ref_enc.Propagate(alpha))
      {
         std::cout << " [FAILED]" << std::endl;
         return false;
      }
   } while (alpha.Inc());
   std::cout << " [OK]" << std::endl;
   return true;
}

