/*
 * Project SUBool
 * Petr Kucera, 2021
 */
/**@file test_dimacs.h
 * Contains tests for DIMACS file parsing (defined in dimacs.h).
 */

#ifndef __TEST_DIMACS_H
#define __TEST_DIMACS_H

#include <iostream>
#include <iterator>
#include <optional>
#include <sstream>
#include <string>

#include "literal.h"
#include "nfliteralset.h"
#include "normalform.h"

namespace SUBool
{
   class TestDIMACS
   {
      template <class SkipFunc>
      bool TestSkip(const std::string &s, std::optional<char> c,
          SkipFunc skip_func) const;
      bool TestSkipBlanks() const;
      bool TestSkipSpaces() const;
      bool TestSkipCommentLines() const;
      bool TestParseLiteral() const;
      bool TestParseLiteralSingle(const std::string &s, bool zero,
          std::optional<Literal> lit, bool ok, std::optional<char> c) const;
      bool TestParseClause() const;
      bool TestParseClauseSingle(const std::string &s,
          std::optional<Clause> clause, bool zero, bool ok,
          std::optional<char> c) const;
      bool TestParseCNFSingle(
          const std::string &s, std::optional<CNF> cnf) const;
      bool TestParseCNF() const;

    public:
      bool RunTests() const;
   };
} // namespace SUBool

template <class SkipFunc>
bool
SUBool::TestDIMACS::TestSkip(
    const std::string &s, std::optional<char> c, SkipFunc skip_func) const
{
   std::cout << "Testing Dimacs::skip_xxx with string \"" << s << "\""
             << std::endl;
   std::stringstream ss(s);
   std::istreambuf_iterator<char> begin(ss), end;
   auto next_pos = skip_func(begin, end);
   if (!c)
   {
      if (next_pos != end)
      {
         std::cout << "[FAILED] expected to reach the end of the string"
                   << std::endl;
         return false;
      }
      std::cout << "[OK]" << std::endl;
      return true;
   }
   if (next_pos == end)
   {
      std::cout << "[FAILED] unexpectedly reached the end of the string"
                << std::endl;
      return false;
   }
   if (*next_pos != c)
   {
      std::cout << "[FAILED] unexpected character" << std::endl;
      return false;
   }
   std::cout << "[OK]" << std::endl;
   return true;
}
#endif
