/*
 * Project SUBool
 * Petr Kucera, 2020
 */
/**@file test_imp_system.h
 * Contains tests of ImpSystem (see imp_system.h)
 */

#ifndef __TEST_IMP_SYSTEM_H
#define __TEST_IMP_SYSTEM_H

#include "impl_system.h"
#include "normalform.h"
#include "unitprop.h"

namespace SUBool
{
   class TestImpSystem
   {
      bool TestClosureSimple() const;
      bool TestMergeBodies() const;
      bool TestHorn() const;
      bool TestRandomHornSingleClosure(const CNF &cnf, UnitPropagator &up,
                                       const VarImplSystem &is,
                                       const std::vector<unsigned> &body) const;
      bool TestRandomHornClosures(const CNF &cnf, const VarImplSystem &is,
                                  const unsigned rounds,
                                  const unsigned body_size) const;
      bool TestRandomHorn(const unsigned cnf_rounds,
                          const unsigned closure_rounds, const unsigned size,
                          const unsigned max_clause_size, const unsigned n_vars,
                          const unsigned body_size) const;
      bool TestRandomHornBodyMinimal(const CNF &cnf,
                                     const VarImplSystem &is) const;
      bool TestRightSaturate(const VarImplSystem &is) const;
      bool TestRightLeftSaturate() const;
      bool TestDualRail() const;
      bool TestGDBasis() const;
      bool TestPropagateEmptyBody() const;
      bool TestBodyMinimal() const;
      bool TestHornRightIrredundant(const CNF &cnf,
                                    const VarImplSystem &is) const;
      bool TestHornLeftIrredundant(const CNF &cnf,
                                   const VarImplSystem &is) const;

    public:
      bool RunTests() const;
   };
} // namespace SUBool

#endif /* end of include guard */
