/*
 * Project SUBool
 * Petr Kucera, 2018
 */
/**@file test_resolve.cpp
 * Contains tests for functions related to resolution.
 */

#include <cstdlib>
#include <ctime>

#include "dimacs.h"
#include "resolve.h"
#include "test_resolve.h"

SUBool::TestResolve::TestResolve() {}

bool
SUBool::TestResolve::TestResolution(const Clause &left, const Clause &right,
    const Clause &resolvent, unsigned res_var) const
{
   std::cout << "Resolving (" << left << ") and (" << right << ")";
   auto res = resolve(left, right);
   if (res.first != resolvent)
   {
      std::cout << " [FAILED]" << std::endl;
      std::cout << "... obtained resolvent (" << res.first
                << ") is not equal to expected (" << resolvent << ")"
                << std::endl;
      return false;
   }
   if (res.second != res_var)
   {
      std::cout << " [FAILED]" << std::endl;
      std::cout << "... obtained resolution variable " << res.second
                << " is not equal to expected " << res_var << std::endl;
      return false;
   }
   std::cout << " [OK]" << std::endl;
   return true;
}

bool
SUBool::TestResolve::RunTests() const
{
   bool success = TestResolution(Dimacs::parse_clause("1 2 0"),
       Dimacs::parse_clause("-1 3 0"), Dimacs::parse_clause("2 3 0"), 1);
   success = success
             && TestResolution(Dimacs::parse_clause("1 2 0"),
                 Dimacs::parse_clause("1 3 0"), Clause(), 0);
   success = success
             && TestResolution(Dimacs::parse_clause("1 3 4 7 -9 -11 0"),
                 Dimacs::parse_clause("-1 3 5 6 -8 -9 11 12 0"), Clause(), 0);
   success = success
             && TestResolution(Dimacs::parse_clause("1 3 4 7 -9 -11 0"),
                 Dimacs::parse_clause("-1 3 5 6 -8 -9 -11 12 0"),
                 Dimacs::parse_clause("3 4 5 6 7 -8 -9 -11 12 0"), 1);
   success = success
             && TestResolution(Dimacs::parse_clause("1 3 4 7 -9 -11 0"),
                 Dimacs::parse_clause("1 3 5 6 -8 9 -11 12 0"),
                 Dimacs::parse_clause("1 3 4 5 6 7 -8 -11 12 0"), 9);
   return success;
}

int
main(int /*argc*/, char * /*argv*/[])
{
   srand((unsigned)time(nullptr));
   SUBool::TestResolve test;
   return !test.RunTests();
}
