
/*
 * Project SUBool
 * Petr Kucera, 2016
 */
/**@file test_unitprop2.cpp
 * Contains tests of UnitPropagatorM (reimplemented.
 */

#include <chrono>
#include <sstream>

#include "dimacs.h"
#include "random_utils.h"
#include "test_unitprop2.h"
#include "unitprop.h"
#include "unitprop2.h"
#include "util.h"

const char *dimacs_simple = "c\n"
                            "c start with comments\n"
                            "c\n"
                            "c\n"
                            "p cnf 5 3\n"
                            "1 -5 4 0\n"
                            "-1 5 3 4 0\n"
                            "-3 -4 0";

const char *dimacs_quinn = "c  quinn.cnf\n"
                           "c\n"
                           "p cnf 16 18\n"
                           "  1    2  0\n"
                           " -2   -4  0\n"
                           "  3    4  0\n"
                           " -4   -5  0\n"
                           "  5   -6  0\n"
                           "  6   -7  0\n"
                           "  6    7  0\n"
                           "  7  -16  0\n"
                           "  8   -9  0\n"
                           " -8  -14  0\n"
                           "  9   10  0\n"
                           "  9  -10  0\n"
                           "-10  -11  0\n"
                           " 10   12  0\n"
                           " 11   12  0\n"
                           " 13   14  0\n"
                           " 14  -15  0\n"
                           " 15   16  0";

const char *dimacs_horn = "c\n"
                          "c A horn formula\n"
                          "c\n"
                          "p cnf 7 8\n"
                          "2 0\n"
                          "-4 0\n"
                          "5 -7 -2 0\n"
                          "-1 3 -4 0\n"
                          "-5 -4 -2 1 0\n"
                          "3 -1 -2 0\n"
                          "5 -2 0\n"
                          "1 -2 -3 -4 0";

const char *amo_product_10 = "c AMO Product\n"
                             "p cnf 16 24\n"
                             "-2 11 0\n"
                             "-1 11 0\n"
                             "-1 -2 0\n"
                             "-3 12 0\n"
                             "-11 12 0\n"
                             "-3 -11 0\n"
                             "-4 13 0\n"
                             "-12 13 0\n"
                             "-4 -12 0\n"
                             "-5 14 0\n"
                             "-13 14 0\n"
                             "-5 -13 0\n"
                             "-6 15 0\n"
                             "-14 15 0\n"
                             "-6 -14 0\n"
                             "-7 16 0\n"
                             "-15 16 0\n"
                             "-7 -15 0\n"
                             "-9 -10 0\n"
                             "-8 -10 0\n"
                             "-8 -9 0\n"
                             "-10 -16 0\n"
                             "-9 -16 0\n"
                             "-8 -16 0\n";

const char *unitprop_cnf = "c\n"
                           "p cnf 6 5\n"
                           "1 -2 0\n"
                           "1 -3 0\n"
                           "2 3 -5 0\n"
                           "2 3 -4 0\n"
                           "1 2 3 4 5 6 0\n";

bool
SUBool::TestUnitPropM::TestDimacsHorn() const
{
   std::cout << "Testing unit propagation on Horn CNF" << std::endl;
   auto cnf = Dimacs::parse_cnf(dimacs_horn);
   std::cout << "Input: " << std::endl;
   std::cout << dimacs_horn << std::endl;

   std::cout << "Trying the empty assignment" << std::endl;
   UnitPropagatorM up(cnf);

   std::cout << "Propagator constructed" << std::endl;
   PartialAssignment alpha(cnf.MaxVariable());
   auto beta = up.Propagate(alpha);

   std::cout << "Consequences: " << beta << std::endl;

   return true;
}

bool
SUBool::TestUnitPropM::TestAmoProduct10() const
{
   std::cout << "Testing unit propagation on AMO Product with n=10"
             << std::endl;
   auto cnf = Dimacs::parse_cnf(amo_product_10);
   std::cout << "Input: " << std::endl;
   std::cout << amo_product_10 << std::endl;

   UnitPropagatorM up(cnf);

   std::cout << "Trying assignments setting one variable to 1" << std::endl;
   for (int i = 1; i <= 10; ++i)
   {
      PartialAssignment alpha(cnf.MaxVariable());
      alpha[i] = PartialValue::TRUE;
      auto beta = up.Propagate(alpha);
      std::cout << "--" << std::endl;
      std::cout << "Assumptions : " << alpha << std::endl;
      std::cout << "Consequences: " << beta << std::endl;
      PartialAssignment gamma(cnf.MaxVariable(), PartialValue::FALSE);
      if (!beta)
      {
         std::cout << "[FAILED]" << std::endl;
         return false;
      }
      gamma[i] = PartialValue::TRUE;
      for (int j = 1; j <= 10; ++j)
      {
         if ((*beta)[j] != gamma[j])
         {
            std::cout << "[FAILED]" << std::endl;
            return false;
         }
      }
      std::cout << "[PASSED]" << std::endl;
   }

   std::cout << "Trying assignments setting two variables to 1" << std::endl;
   for (int i = 1; i <= 10; ++i)
   {
      for (int j = 1; j <= 10; ++j)
      {
         if (j == i)
         {
            continue;
         }
         PartialAssignment alpha(cnf.MaxVariable());
         alpha[i] = PartialValue::TRUE;
         alpha[j] = PartialValue::TRUE;
         std::cout << "--" << std::endl;
         auto beta = up.Propagate(alpha);
         std::cout << "Result: " << beta;
         if (beta)
         {
            std::cout << " [FAILED]" << std::endl;
            return false;
         }
         std::cout << " [PASSED]" << std::endl;
      }
   }

   return true;
}

bool
SUBool::TestUnitPropM::TestUnitPropCNF() const
{
   std::cout << "Testing unit propagation on CNF" << std::endl;
   auto cnf = Dimacs::parse_cnf(unitprop_cnf);
   std::cout << "Input: " << std::endl;
   std::cout << unitprop_cnf << std::endl;

   std::cout << "Trying the assignment with assumptions -1" << std::endl;
   PartialAssignment alpha(cnf.MaxVariable());
   alpha[1] = PartialValue::FALSE;
   // alpha[6] = PartialValue::FALSE;

   UnitPropagatorM up(cnf);
   auto beta = up.Propagate(alpha);
   std::cout << "Consequences: " << beta << std::endl;
   if (!beta)
   {
      std::cout << "[FAILED]" << std::endl;
      return false;
   }
   PartialAssignment gamma(cnf.MaxVariable(), PartialValue::FALSE);
   gamma[cnf.MaxVariable()] = PartialValue::TRUE;
   std::cout << "Gamma: " << gamma << std::endl;
   for (unsigned k = 1; k <= cnf.MaxVariable(); ++k)
   {
      if ((*beta)[k] != gamma[k])
      {
         std::cout << (*beta)[k] << "!=" << gamma[k] << " k: " << k
                   << std::endl;
         std::cout << "[FAILED]" << std::endl;
         return false;
      }
   }
   std::cout << "[PASSED]" << std::endl;
   return true;
}

bool
SUBool::TestUnitPropM::TestSingleRandomCNF(const CNF &cnf,
    const unsigned assump_rounds, const unsigned assump_size) const
{
   UnitPropagator up(&cnf);
   UnitPropagatorM up_m(cnf);
   for (unsigned round = 0; round < assump_rounds; ++round)
   {
      auto assump_lits =
          random_literal_vector_with_size(assump_size, cnf.MaxVariable());
      PartialAssignment alpha(cnf.MaxVariable(), assump_lits);
      PartialAssignment clos(0);
      auto confl = up.Propagate(alpha, &clos);
      auto clos_m = up_m.Propagate(assump_lits);
      if ((clos_m && confl >= 0) || (!clos_m && confl < 0))
      {
         std::cout << "[FAILED] Different results on random CNF" << std::endl;
         std::cout << "confl: " << confl << std::endl;
         std::cout << "clos_m: " << clos_m << std::endl;
         std::cout << "CNF" << std::endl;
         std::cout << cnf << std::endl;
         std::cout << "c assumptions" << std::endl;
         for (const auto &lit : assump_lits)
         {
            std::cout << lit << " 0" << std::endl;
         }
         std::cout << "c assumption pa: " << alpha << std::endl;
         return false;
      }
      if (clos_m && *clos_m != clos)
      {
         std::cout << "[FAILED] Different closures on random CNF" << std::endl;
         return false;
      }
   }
   return true;
}

bool
SUBool::TestUnitPropM::TestRandomCNF(const unsigned rounds,
    const unsigned n_clauses, const unsigned n_vars,
    const unsigned assump_rounds, const unsigned assump_size) const
{
   std::cout << "Testing propagation on random formulas" << std::endl;
   for (unsigned round = 0; round < rounds; ++round)
   {
      auto cnf = random_cnf(n_clauses, n_vars);
      if (!TestSingleRandomCNF(cnf, assump_rounds, assump_size))
      {
         std::cout << "[FAILED] round " << round << std::endl;
         return false;
      }
   }
   std::cout << "[ALL OK]" << std::endl;
   return true;
}

bool
SUBool::TestUnitPropM::RunTests() const
{
   bool success = TestDimacsHorn();
   success = success && TestAmoProduct10();
   success = success && TestUnitPropCNF();
   success = success && TestRandomCNF(10, 50, 20, 100, 5);
   //   bool success=TestStdIn();
   return success;
}

int
main(int /*argc*/, char * /*argv*/[])
{
   srand(time(NULL));
   SUBool::TestUnitPropM test;
   return !test.RunTests();
}
